@@ -39,6 +39,7 @@
 #include <dbus/dbus-spawn.h>
 #include <dbus/dbus-timeout.h>
 #include <dbus/dbus-sysdeps.h>
+#include <dbus/dbus-test-tap.h>
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
 #endif
@@ -847,6 +848,42 @@ populate_environment (BusActivation *activation)
   retval = _dbus_hash_table_from_array (activation->environment, environment, '=');
   dbus_free_string_array (environment);
 
+  /*
+   * These environment variables are set by systemd for the dbus-daemon
+   * itself, and are not applicable to our child processes.
+   *
+   * Of the other environment variables listed in systemd.exec(5):
+   *
+   * - XDG_RUNTIME_DIR, XDG_SESSION_ID, XDG_SEAT, XDG_VTNR: Properties of
+   *   the session and equally true for the activated service, should not
+   *   be reset
+   * - PATH, LANG, USER, LOGNAME, HOME, SHELL, MANAGERPID: Equally true for
+   *   the activated service, should not be reset
+   * - TERM, WATCHDOG_*: Should not be set for dbus-daemon, so not applicable
+   * - MAINPID, SERVICE_RESULT, EXIT_CODE, EXIT_STATUS: Not set for ExecStart,
+   *   so not applicable
+   */
+
+  /* We give activated services their own Journal stream to avoid their
+   * logging being attributed to dbus-daemon */
+  _dbus_hash_table_remove_string (activation->environment, "JOURNAL_STREAM");
+
+  /* This is dbus-daemon's listening socket, not the activatable service's */
+  _dbus_hash_table_remove_string (activation->environment, "LISTEN_FDNAMES");
+  _dbus_hash_table_remove_string (activation->environment, "LISTEN_FDS");
+  _dbus_hash_table_remove_string (activation->environment, "LISTEN_PID");
+
+  /* This is dbus-daemon's status notification, not the activatable service's
+   * (and NotifyAccess wouldn't let it write here anyway) */
+  _dbus_hash_table_remove_string (activation->environment, "NOTIFY_SOCKET");
+
+  /* This identifies the dbus-daemon invocation. Whether it should be
+   * inherited by "smaller" services isn't entirely clear-cut, but not
+   * inheriting it makes traditional D-Bus activation under systemd a
+   * little more consistent with systemd activation.
+   * https://lists.freedesktop.org/archives/systemd-devel/2018-March/040467.html */
+  _dbus_hash_table_remove_string (activation->environment, "INVOCATION_ID");
+
   return retval;
 }
 
@@ -877,9 +914,8 @@ bus_activation_reload (BusActivation     *activation,
       goto failed;
     }
 
-  _dbus_list_foreach (&activation->directories,
-                      (DBusForeachFunction) bus_service_directory_unref, NULL);
-  _dbus_list_clear (&activation->directories);
+  _dbus_list_clear_full (&activation->directories,
+                         (DBusFreeFunction) bus_service_directory_unref);
 
   link = _dbus_list_get_first_link (directories);
   while (link != NULL)
@@ -1026,9 +1062,8 @@ bus_activation_unref (BusActivation *activation)
   if (activation->pending_activations)
     _dbus_hash_table_unref (activation->pending_activations);
 
-  _dbus_list_foreach (&activation->directories,
-                      (DBusForeachFunction) bus_service_directory_unref, NULL);
-  _dbus_list_clear (&activation->directories);
+  _dbus_list_clear_full (&activation->directories,
+                         (DBusFreeFunction) bus_service_directory_unref);
 
   if (activation->environment)
     _dbus_hash_table_unref (activation->environment);
@@ -1733,6 +1768,21 @@ child_setup (void *user_data)
 }
 
 
+/*
+ * Try to activate the given service.
+ *
+ * connection is the connection requesting that the service be started,
+ * or NULL if the activation was caused by the dbus-daemon itself (when
+ * systemd activation waits for systemd to connect to us, or when calling
+ * SetEnvironment on systemd).
+ *
+ * auto_activation is TRUE if we are carrying out auto-starting (we are
+ * activating a service automatically in order to deliver a message to it)
+ * or FALSE if we are starting the service explicitly (as for
+ * StartServiceByName).
+ *
+ * activation_message is the message that caused this activation.
+ */
 dbus_bool_t
 bus_activation_activate_service (BusActivation  *activation,
                                  DBusConnection *connection,
@@ -1758,6 +1808,10 @@ bus_activation_activate_service (BusActivation  *activation,
   int limit;
   DBusSpawnFlags flags = DBUS_SPAWN_NONE;
 
+  _dbus_assert (activation != NULL);
+  _dbus_assert (transaction != NULL);
+  _dbus_assert (activation_message != NULL);
+  _dbus_assert (service_name != NULL);
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
 
   limit = bus_context_get_max_pending_activations (activation->context);
@@ -2219,6 +2273,11 @@ bus_activation_activate_service (BusActivation  *activation,
 
   dbus_error_init (&tmp_error);
 
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
+  if (bus_context_get_quiet_log (activation->context))
+    flags |= DBUS_SPAWN_SILENCE_OUTPUT;
+#endif
+
   if (bus_context_get_using_syslog (activation->context))
     flags |= DBUS_SPAWN_REDIRECT_OUTPUT;
 
@@ -2530,7 +2589,8 @@ typedef struct
 } CheckData;
 
 static dbus_bool_t
-check_func (void *data)
+check_func (void        *data,
+            dbus_bool_t  have_memory)
 {
   CheckData          *d;
   BusActivationEntry *entry;
@@ -2574,10 +2634,10 @@ do_test (const char *description, dbus_bool_t oom_test, CheckData *data)
   if (oom_test)
     err = !_dbus_test_oom_handling (description, check_func, data);
   else
-    err = !check_func (data);
+    err = !check_func (data, TRUE);
 
   if (err)
-    _dbus_assert_not_reached ("Test failed");
+    _dbus_test_fatal ("Test failed");
 
   return TRUE;
 }
@@ -2698,7 +2758,7 @@ bus_activation_service_reload_test (const DBusString *test_data_dir)
 
   /* Do normal tests */
   if (!init_service_reload_test (&directory))
-    _dbus_assert_not_reached ("could not initiate service reload test");
+    _dbus_test_fatal ("could not initiate service reload test");
 
   if (!do_service_reload_test (test_data_dir, &directory, FALSE))
     {
@@ -2710,7 +2770,7 @@ bus_activation_service_reload_test (const DBusString *test_data_dir)
 
   /* Do OOM tests */
   if (!init_service_reload_test (&directory))
-    _dbus_assert_not_reached ("could not initiate service reload test");
+    _dbus_test_fatal ("could not initiate service reload test");
 
   if (!do_service_reload_test (test_data_dir, &directory, TRUE))
     {
