@@ -28,6 +28,7 @@
 #include "utils.h"
 #include <dbus/dbus-list.h>
 #include <dbus/dbus-internals.h>
+#include <dbus/dbus-test-tap.h>
 #include <string.h>
 
 /**
@@ -121,13 +122,7 @@ bus_config_parser_unref (BusConfigParser *parser)
   _dbus_string_free (&parser->user);
   _dbus_string_free (&parser->service_helper);
   _dbus_string_free (&parser->bus_type);
-
-  _dbus_list_foreach (&parser->service_dirs,
-                      (DBusForeachFunction) dbus_free,
-                      NULL);
-
-  _dbus_list_clear (&parser->service_dirs);
-
+  _dbus_list_clear_full (&parser->service_dirs, dbus_free);
   dbus_free (parser);
 }
 
@@ -162,6 +157,7 @@ bus_config_parser_start_element (BusConfigParser   *parser,
 
   switch (parser->type)
     {
+    case ELEMENT_SERVICEDIR:
     case ELEMENT_SERVICEHELPER:
     case ELEMENT_USER:
     case ELEMENT_CONFIGTYPE:
@@ -185,6 +181,26 @@ bus_config_parser_start_element (BusConfigParser   *parser,
         break;
       }
 
+    case ELEMENT_NONE:
+    case ELEMENT_BUSCONFIG:
+    case ELEMENT_INCLUDE:
+    case ELEMENT_LISTEN:
+    case ELEMENT_AUTH:
+    case ELEMENT_POLICY:
+    case ELEMENT_LIMIT:
+    case ELEMENT_ALLOW:
+    case ELEMENT_DENY:
+    case ELEMENT_FORK:
+    case ELEMENT_PIDFILE:
+    case ELEMENT_INCLUDEDIR:
+    case ELEMENT_SELINUX:
+    case ELEMENT_ASSOCIATE:
+    case ELEMENT_STANDARD_SESSION_SERVICEDIRS:
+    case ELEMENT_KEEP_UMASK:
+    case ELEMENT_SYSLOG:
+    case ELEMENT_ALLOW_ANONYMOUS:
+    case ELEMENT_APPARMOR:
+      /* fall through */
     default:
       {
         /* we really don't care about the others... */
@@ -286,6 +302,28 @@ bus_config_parser_content (BusConfigParser   *parser,
           }
       }
       break;
+
+    case ELEMENT_NONE:
+    case ELEMENT_BUSCONFIG:
+    case ELEMENT_INCLUDE:
+    case ELEMENT_LISTEN:
+    case ELEMENT_AUTH:
+    case ELEMENT_POLICY:
+    case ELEMENT_LIMIT:
+    case ELEMENT_ALLOW:
+    case ELEMENT_DENY:
+    case ELEMENT_FORK:
+    case ELEMENT_PIDFILE:
+    case ELEMENT_INCLUDEDIR:
+    case ELEMENT_SELINUX:
+    case ELEMENT_ASSOCIATE:
+    case ELEMENT_STANDARD_SESSION_SERVICEDIRS:
+    case ELEMENT_STANDARD_SYSTEM_SERVICEDIRS:
+    case ELEMENT_KEEP_UMASK:
+    case ELEMENT_SYSLOG:
+    case ELEMENT_ALLOW_ANONYMOUS:
+    case ELEMENT_APPARMOR:
+      /* fall through */
     default:
       {
         /* we don't care about the others... really */
@@ -324,8 +362,11 @@ bus_config_parser_get_type (BusConfigParser *parser)
   return _dbus_string_get_const_data (&parser->bus_type);
 }
 
+/*
+ * @returns A list of strings, owned by the BusConfigParser
+ */
 DBusList**
-bus_config_parser_get_service_dirs (BusConfigParser *parser)
+bus_config_parser_get_service_paths (BusConfigParser *parser)
 {
   return &parser->service_dirs;
 }
@@ -354,7 +395,7 @@ check_return_values (const DBusString *full_path)
   dbus_error_init (&error);
   retval = FALSE;
 
-  printf ("Testing values from: %s\n", _dbus_string_get_const_data (full_path));
+  _dbus_test_diag ("Testing values from: %s", _dbus_string_get_const_data (full_path));
 
   parser = bus_config_load (full_path, TRUE, NULL, &error);
   if (parser == NULL)
@@ -380,7 +421,7 @@ check_return_values (const DBusString *full_path)
       _dbus_warn ("User was invalid; '%s'!", user);
       goto finish;
     }
-  printf ("    <user>dbus</user> OKAY!\n");  
+  _dbus_test_diag ("    <user>dbus</user> OKAY!");
 #endif
   
   /* check type return value is okay */
@@ -395,16 +436,16 @@ check_return_values (const DBusString *full_path)
       _dbus_warn ("Type was invalid; '%s'!", user);
       goto finish;
     }
-  printf ("    <type>system</type> OKAY!\n");
+  _dbus_test_diag ("    <type>system</type> OKAY!");
 
   /* check dirs return value is okay */
-  dirs = bus_config_parser_get_service_dirs (parser);
+  dirs = bus_config_parser_get_service_paths (parser);
   if (dirs == NULL)
     {
       _dbus_warn ("Service dirs are NULL!");
       goto finish;
     }
-  printf ("    <standard_system_service_dirs/> OKAY!\n");
+  _dbus_test_diag ("    <standard_system_service_dirs/> OKAY!");
   /* NOTE: We tested the specific return values in the config-parser tests */
 
   /* woohoo! */
@@ -497,20 +538,20 @@ process_test_valid_subdir (const DBusString *test_base_dir,
   dir = NULL;
 
   if (!_dbus_string_init (&test_directory))
-    _dbus_assert_not_reached ("didn't allocate test_directory");
+    _dbus_test_fatal ("didn't allocate test_directory");
 
   _dbus_string_init_const (&filename, subdir);
 
   if (!_dbus_string_copy (test_base_dir, 0,
                           &test_directory, 0))
-    _dbus_assert_not_reached ("couldn't copy test_base_dir to test_directory");
+    _dbus_test_fatal ("couldn't copy test_base_dir to test_directory");
 
   if (!_dbus_concat_dir_and_file (&test_directory, &filename))
-    _dbus_assert_not_reached ("couldn't allocate full path");
+    _dbus_test_fatal ("couldn't allocate full path");
 
   _dbus_string_free (&filename);
   if (!_dbus_string_init (&filename))
-    _dbus_assert_not_reached ("didn't allocate filename string");
+    _dbus_test_fatal ("didn't allocate filename string");
 
   dbus_error_init (&error);
   dir = _dbus_directory_open (&test_directory, &error);
@@ -524,11 +565,11 @@ process_test_valid_subdir (const DBusString *test_base_dir,
     }
 
   if (validity == VALID)
-    printf ("Testing valid files:\n");
+    _dbus_test_diag ("Testing valid files:");
   else if (validity == INVALID)
-    printf ("Testing invalid files:\n");
+    _dbus_test_diag ("Testing invalid files:");
   else
-    printf ("Testing unknown files:\n");
+    _dbus_test_diag ("Testing unknown files:");
 
  next:
   while (_dbus_directory_get_next_file (dir, &filename, &error))
@@ -537,13 +578,13 @@ process_test_valid_subdir (const DBusString *test_base_dir,
       LoaderOomData d;
 
       if (!_dbus_string_init (&full_path))
-        _dbus_assert_not_reached ("couldn't init string");
+        _dbus_test_fatal ("couldn't init string");
 
       if (!_dbus_string_copy (&test_directory, 0, &full_path, 0))
-        _dbus_assert_not_reached ("couldn't copy dir to full_path");
+        _dbus_test_fatal ("couldn't copy dir to full_path");
 
       if (!_dbus_concat_dir_and_file (&full_path, &filename))
-        _dbus_assert_not_reached ("couldn't concat file to dir");
+        _dbus_test_fatal ("couldn't concat file to dir");
 
       if (!_dbus_string_ends_with_c_str (&full_path, ".conf"))
         {
@@ -553,7 +594,7 @@ process_test_valid_subdir (const DBusString *test_base_dir,
           goto next;
         }
 
-      printf ("    %s\n", _dbus_string_get_const_data (&filename));
+      _dbus_test_diag ("    %s", _dbus_string_get_const_data (&filename));
 
       _dbus_verbose (" expecting %s\n",
                      validity == VALID ? "valid" :
@@ -569,8 +610,8 @@ process_test_valid_subdir (const DBusString *test_base_dir,
        */
       /* if (!_dbus_test_oom_handling ("config-loader", check_loader_oom_func, &d)) */
       if (!check_loader_oom_func (&d))
-        _dbus_assert_not_reached ("test failed");
-      
+        _dbus_test_fatal ("test failed");
+
       _dbus_string_free (&full_path);
     }
 
@@ -649,14 +690,14 @@ check_file_valid (DBusString *full_path,
   dbus_bool_t retval;
 
   if (validity == VALID)
-    printf ("Testing valid file:\n");
+    _dbus_test_diag ("Testing valid file:");
   else if (validity == INVALID)
-    printf ("Testing invalid file:\n");
+    _dbus_test_diag ("Testing invalid file:");
   else
-    printf ("Testing unknown file:\n");
+    _dbus_test_diag ("Testing unknown file:");
 
   /* print the filename, just so we match the other output */
-  printf ("    %s\n", _dbus_string_get_const_data (full_path));
+  _dbus_test_diag ("    %s", _dbus_string_get_const_data (full_path));
 
   /* only test one file */
   retval = do_load (full_path, validity, TRUE);
@@ -675,7 +716,7 @@ bus_config_parser_trivial_test (const DBusString *test_data_dir)
   if (test_data_dir == NULL ||
       _dbus_string_get_length (test_data_dir) == 0)
     {
-      printf ("No test data\n");
+      _dbus_test_diag ("No test data");
       return TRUE;
     }
   
