@@ -23,6 +23,8 @@
 
 #include <config.h>
 #include "connection.h"
+
+#include "containers.h"
 #include "dispatch.h"
 #include "policy.h"
 #include "services.h"
@@ -106,6 +108,7 @@ typedef struct
   long connection_tv_sec;  /**< Time when we connected (seconds component) */
   long connection_tv_usec; /**< Time when we connected (microsec component) */
   int stamp;               /**< connections->stamp last time we were traversed */
+  BusExtraHeaders want_headers;
 
 #ifdef DBUS_ENABLE_STATS
   int peak_match_rules;
@@ -306,6 +309,9 @@ bus_connection_disconnected (DBusConnection *connection)
       d->link_in_monitors = NULL;
     }
 
+  bus_containers_remove_connection (bus_context_get_containers (d->connections->context),
+                                    connection);
+
   if (d->link_in_connection_list != NULL)
     {
       if (d->name != NULL)
@@ -537,6 +543,9 @@ bus_connections_unref (BusConnections *connections)
 
       _dbus_assert (connections->n_incomplete == 0);
 
+      /* drop all monitors */
+      _dbus_list_clear (&connections->monitors);
+
       /* drop all real connections */
       while (connections->completed != NULL)
         {
@@ -552,10 +561,6 @@ bus_connections_unref (BusConnections *connections)
 
       _dbus_assert (connections->n_completed == 0);
 
-      /* disconnecting all the connections should have emptied the list of
-       * monitors (each link is removed in bus_connection_disconnected) */
-      _dbus_assert (connections->monitors == NULL);
-
       bus_expire_list_free (connections->pending_replies);
       
       _dbus_loop_remove_timeout (bus_context_get_loop (connections->context),
@@ -580,31 +585,48 @@ cache_peer_loginfo_string (BusConnectionData *d,
                            DBusConnection    *connection)
 {
   DBusString loginfo_buf;
-  unsigned long uid;
-  unsigned long pid;
-  char *windows_sid = NULL, *security_label = NULL;
+  dbus_uid_t uid = DBUS_UID_UNSET;
+  dbus_pid_t pid = DBUS_PID_UNSET;
+  const char *windows_sid = NULL;
+  const char *security_label = NULL;
   dbus_bool_t prev_added;
+  const char *container = NULL;
+  const char *container_type = NULL;
+  const char *container_name = NULL;
+  DBusCredentials *credentials;
 
   if (!_dbus_string_init (&loginfo_buf))
     return FALSE;
-  
+
+  credentials = _dbus_connection_get_credentials (connection);
+
   prev_added = FALSE;
-  if (dbus_connection_get_unix_user (connection, &uid))
+
+  if (credentials != NULL)
     {
-      if (!_dbus_string_append_printf (&loginfo_buf, "uid=%ld", uid))
+      uid = _dbus_credentials_get_unix_uid (credentials);
+      pid = _dbus_credentials_get_pid (credentials);
+      windows_sid = _dbus_credentials_get_windows_sid (credentials);
+      security_label = _dbus_credentials_get_linux_security_label (credentials);
+    }
+
+  if (uid != DBUS_UID_UNSET)
+    {
+      if (!_dbus_string_append_printf (&loginfo_buf, "uid=" DBUS_UID_FORMAT, uid))
         goto oom;
       else
         prev_added = TRUE;
     }
 
-  if (dbus_connection_get_unix_process_id (connection, &pid))
+  if (pid != DBUS_PID_UNSET)
     {
       if (prev_added)
         {
           if (!_dbus_string_append_byte (&loginfo_buf, ' '))
             goto oom;
         }
-      if (!_dbus_string_append_printf (&loginfo_buf, "pid=%ld comm=\"", pid))
+      if (!_dbus_string_append_printf (&loginfo_buf,
+                                       "pid=" DBUS_PID_FORMAT " comm=\"", pid))
         goto oom;
       /* Ignore errors here; we may not have permissions to read the
        * proc file. */
@@ -615,7 +637,7 @@ cache_peer_loginfo_string (BusConnectionData *d,
         prev_added = TRUE;
     }
 
-  if (dbus_connection_get_windows_user (connection, &windows_sid))
+  if (windows_sid != NULL)
     {
       dbus_bool_t did_append;
 
@@ -627,15 +649,13 @@ cache_peer_loginfo_string (BusConnectionData *d,
 
       did_append = _dbus_string_append_printf (&loginfo_buf,
                                                "sid=\"%s\"", windows_sid);
-      dbus_free (windows_sid);
-      windows_sid = NULL;
       if (!did_append)
         goto oom;
       else
         prev_added = TRUE;
     }
 
-  if (_dbus_connection_get_linux_security_label (connection, &security_label))
+  if (security_label != NULL)
     {
       dbus_bool_t did_append;
 
@@ -647,8 +667,30 @@ cache_peer_loginfo_string (BusConnectionData *d,
 
       did_append = _dbus_string_append_printf (&loginfo_buf,
                                                "label=\"%s\"", security_label);
-      dbus_free (security_label);
-      security_label = NULL;
+      if (!did_append)
+        goto oom;
+      else
+        prev_added = TRUE;
+    }
+
+  /* This does have to come from the connection, not the credentials */
+  if (bus_containers_connection_is_contained (connection, &container,
+                                              &container_type,
+                                              &container_name))
+    {
+      dbus_bool_t did_append;
+
+      if (prev_added)
+        {
+          if (!_dbus_string_append_byte (&loginfo_buf, ' '))
+            goto oom;
+        }
+
+      did_append = _dbus_string_append_printf (&loginfo_buf,
+                                               "container=%s %s=\"%s\")",
+                                               container,
+                                               container_type,
+                                               container_name);
       if (!did_append)
         goto oom;
       else
@@ -663,11 +705,6 @@ cache_peer_loginfo_string (BusConnectionData *d,
   return TRUE;
 oom:
    _dbus_string_free (&loginfo_buf);
-   if (security_label != NULL)
-     dbus_free (security_label);
-   if (windows_sid != NULL)
-     dbus_free (windows_sid);
-
    return FALSE;
 }
 
@@ -1002,11 +1039,43 @@ bus_connection_get_unix_groups  (DBusConnection   *connection,
                                  int              *n_groups,
                                  DBusError        *error)
 {
+  /* Assigning dbus_gid_t to unsigned long is lossless (in fact
+   * they are the same type) */
+  _DBUS_STATIC_ASSERT (sizeof (unsigned long) == sizeof (dbus_gid_t));
+
+  const dbus_gid_t *groups_borrowed = NULL;
+  DBusCredentials *credentials;
   unsigned long uid;
+  size_t n = 0;
 
   *groups = NULL;
   *n_groups = 0;
 
+  credentials = _dbus_connection_get_credentials (connection);
+
+  if (credentials != NULL &&
+      _dbus_credentials_get_unix_gids (credentials, &groups_borrowed, &n))
+    {
+      size_t i;
+
+      /* We got the group IDs from SO_PEERGROUPS or equivalent - no
+       * need to ask NSS */
+
+      *n_groups = n;
+      *groups = dbus_new (unsigned long, n);
+
+      if (*groups == NULL)
+        {
+          BUS_SET_OOM (error);
+          return FALSE;
+        }
+
+      for (i = 0; i < n; i++)
+        (*groups)[i] = groups_borrowed[i];
+
+      return TRUE;
+    }
+
   if (dbus_connection_get_unix_user (connection, &uid))
     {
       if (!_dbus_unix_groups_from_uid (uid, groups, n_groups))
@@ -2281,7 +2350,7 @@ bus_transaction_capture (BusTransaction *transaction,
     {
       DBusConnection *recipient = link->data;
 
-      if (!bus_transaction_send (transaction, recipient, message))
+      if (!bus_transaction_send (transaction, sender, recipient, message))
         goto out;
     }
 
@@ -2350,21 +2419,6 @@ bus_transaction_send_from_driver (BusTransaction *transaction,
   if (!dbus_message_set_sender (message, DBUS_SERVICE_DBUS))
     return FALSE;
 
-  /* Make sure the message has a non-zero serial number, otherwise
-   * bus_transaction_capture_error_reply() will not be able to mock up
-   * a corresponding reply for it. Normally this would be delayed until
-   * the first time we actually send the message out from a
-   * connection, when the transaction is committed, but that's too late
-   * in this case.
-   */
-  if (dbus_message_get_serial (message) == 0)
-    {
-      dbus_uint32_t next_serial;
-
-      next_serial = _dbus_connection_get_next_client_serial (connection);
-      dbus_message_set_serial (message, next_serial);
-    }
-
   if (bus_connection_is_active (connection))
     {
       if (!dbus_message_set_destination (message,
@@ -2405,12 +2459,13 @@ bus_transaction_send_from_driver (BusTransaction *transaction,
       return TRUE;
     }
 
-  return bus_transaction_send (transaction, connection, message);
+  return bus_transaction_send (transaction, NULL, connection, message);
 }
 
 dbus_bool_t
 bus_transaction_send (BusTransaction *transaction,
-                      DBusConnection *connection,
+                      DBusConnection *sender,
+                      DBusConnection *destination,
                       DBusMessage    *message)
 {
   MessageToSend *to_send;
@@ -2427,24 +2482,44 @@ bus_transaction_send (BusTransaction *transaction,
                  dbus_message_get_member (message) : "(unset)",
                  dbus_message_get_error_name (message) ?
                  dbus_message_get_error_name (message) : "(unset)",
-                 dbus_connection_get_is_connected (connection) ?
+                 dbus_connection_get_is_connected (destination) ?
                  "" : " (disconnected)");
 
   _dbus_assert (dbus_message_get_sender (message) != NULL);
   
-  if (!dbus_connection_get_is_connected (connection))
-    return TRUE; /* silently ignore disconnected connections */
+  if (!dbus_connection_get_is_connected (destination))
+    return TRUE; /* silently ignore disconnected destinations */
   
-  d = BUS_CONNECTION_DATA (connection);
+  d = BUS_CONNECTION_DATA (destination);
   _dbus_assert (d != NULL);
-  
+
+  /* You might think that this is too late to be setting header fields,
+   * because the message is locked before sending - but remember that
+   * the message isn't actually queued to be sent (and hence locked)
+   * until we know we have enough memory for the entire transaction,
+   * and that doesn't happen until we know all the recipients.
+   * So this is about the last possible time we could edit the header. */
+  if ((d->want_headers & BUS_EXTRA_HEADERS_CONTAINER_INSTANCE) &&
+      dbus_message_get_container_instance (message) == NULL)
+    {
+      const char *path;
+
+      if (sender == NULL ||
+          !bus_containers_connection_is_contained (sender, &path,
+                                                   NULL, NULL))
+        path = "/";
+
+      if (!dbus_message_set_container_instance (message, path))
+        return FALSE;
+    }
+
   to_send = dbus_new (MessageToSend, 1);
   if (to_send == NULL)
     {
       return FALSE;
     }
 
-  to_send->preallocated = dbus_connection_preallocate_send (connection);
+  to_send->preallocated = dbus_connection_preallocate_send (destination);
   if (to_send->preallocated == NULL)
     {
       dbus_free (to_send);
@@ -2459,13 +2534,13 @@ bus_transaction_send (BusTransaction *transaction,
   
   if (!_dbus_list_prepend (&d->transaction_messages, to_send))
     {
-      message_to_send_free (connection, to_send);
+      message_to_send_free (destination, to_send);
       return FALSE;
     }
 
   _dbus_verbose ("prepended message\n");
   
-  /* See if we already had this connection in the list
+  /* See if we already had this destination in the list
    * for this transaction. If we have a pending message,
    * then we should already be in transaction->connections
    */
@@ -2485,10 +2560,10 @@ bus_transaction_send (BusTransaction *transaction,
 
   if (link == NULL)
     {
-      if (!_dbus_list_prepend (&transaction->connections, connection))
+      if (!_dbus_list_prepend (&transaction->connections, destination))
         {
           _dbus_list_remove (&d->transaction_messages, to_send);
-          message_to_send_free (connection, to_send);
+          message_to_send_free (destination, to_send);
           return FALSE;
         }
     }
@@ -2897,3 +2972,15 @@ bus_connection_be_monitor (DBusConnection  *connection,
 
   return TRUE;
 }
+
+void
+bus_connection_request_headers (DBusConnection  *connection,
+                                BusExtraHeaders  headers)
+{
+  BusConnectionData *d;
+
+  d = BUS_CONNECTION_DATA (connection);
+  _dbus_assert (d != NULL);
+
+  d->want_headers |= headers;
+}
