@@ -28,6 +28,12 @@
 #include <dbus/dbus-list.h>
 #include "bus.h"
 
+typedef enum
+{
+  BUS_EXTRA_HEADERS_CONTAINER_INSTANCE = (1 << 0),
+  BUS_EXTRA_HEADERS_NONE = 0
+} BusExtraHeaders;
+
 typedef dbus_bool_t (* BusConnectionForeachFunction) (DBusConnection *connection, 
                                                       void           *data);
 
@@ -57,6 +63,8 @@ BusSELinuxID*   bus_connection_get_selinux_id     (DBusConnection
 BusAppArmorConfinement* bus_connection_dup_apparmor_confinement (DBusConnection *connection);
 dbus_bool_t     bus_connections_check_limits      (BusConnections               *connections,
                                                    DBusConnection               *requesting_completion,
+                                                   const char                  **limit_name_out,
+                                                   int                          *limit_out,
                                                    DBusError                    *error);
 void            bus_connections_expire_incomplete (BusConnections               *connections);
 
@@ -82,6 +90,9 @@ dbus_bool_t bus_connection_preallocate_oom_error (DBusConnection *connection);
 void        bus_connection_send_oom_error        (DBusConnection *connection,
                                                   DBusMessage    *in_reply_to);
 
+void        bus_connection_request_headers       (DBusConnection  *connection,
+                                                  BusExtraHeaders  headers);
+
 /* called by signals.c */
 dbus_bool_t bus_connection_add_match_rule      (DBusConnection *connection,
                                                 BusMatchRule   *rule);
@@ -130,12 +141,15 @@ typedef void (* BusTransactionCancelFunction) (void *data);
 BusTransaction* bus_transaction_new              (BusContext                   *context);
 BusContext*     bus_transaction_get_context      (BusTransaction               *transaction);
 dbus_bool_t     bus_transaction_send             (BusTransaction               *transaction,
-                                                  DBusConnection               *connection,
+                                                  DBusConnection               *sender,
+                                                  DBusConnection               *destination,
                                                   DBusMessage                  *message);
 dbus_bool_t     bus_transaction_capture          (BusTransaction               *transaction,
                                                   DBusConnection               *connection,
+                                                  DBusConnection               *addressed_recipient,
                                                   DBusMessage                  *message);
 dbus_bool_t     bus_transaction_capture_error_reply (BusTransaction            *transaction,
+                                                  DBusConnection               *addressed_recipient,
                                                   const DBusError              *error,
                                                   DBusMessage                  *in_reply_to);
 dbus_bool_t     bus_transaction_send_from_driver (BusTransaction               *transaction,
