@@ -378,16 +378,12 @@ parse_comment_or_blank (BusDesktopFileParser *parser)
 static dbus_bool_t
 is_valid_section_name (const char *name)
 {
-  /* 5. Group names may contain all ASCII characters except for control characters and '[' and ']'.
-   *
-   * We don't use isprint() here because it's locale-dependent. ASCII
-   * characters <= 0x1f and 0x7f are control characters, and bytes with
-   * values >= 0x80 aren't ASCII. 0x20 is a space, which we must allow,
-   * not least because DBUS_SERVICE_SECTION contains one. */
+  /* 5. Group names may contain all ASCII characters except for control characters and '[' and ']'. */
 
   while (*name)
     {
-      if (*name <= 0x1f || *name >= 0x7f || *name  == '[' || *name == ']')
+      if (!((*name >= 'A' && *name <= 'Z') || (*name >= 'a' || *name <= 'z') ||
+	    *name == '\n' || *name == '\t'))
 	return FALSE;
       
       name++;
@@ -602,27 +598,6 @@ report_error (BusDesktopFileParser *parser,
                     "Error at line %d: %s\n", parser->line_num, message);
 }
 
-#if 0
-static void
-dump_desktop_file (BusDesktopFile *file)
-{
-  int i;
-
-  for (i = 0; i < file->n_sections; i++)
-    {
-      int j;
-      
-      printf ("[%s]\n", file->sections[i].section_name);
-
-      for (j = 0; j < file->sections[i].n_lines; j++)
-	{
-	  printf ("%s=%s\n", file->sections[i].lines[j].key,
-		  file->sections[i].lines[j].value);
-	}
-    }
-}
-#endif
-
 BusDesktopFile*
 bus_desktop_file_load (DBusString *filename,
 		       DBusError  *error)
