@@ -26,6 +26,7 @@
 #include "test.h"
 #include <dbus/dbus-internals.h>
 #include <dbus/dbus-mainloop.h>
+#include <dbus/dbus-test-tap.h>
 #include <dbus/dbus-timeout.h>
 
 struct BusExpireList
@@ -362,7 +363,7 @@ bus_expire_list_test (const DBusString *test_data_dir)
   item->item.added_tv_sec = tv_sec;
   item->item.added_tv_usec = tv_usec;
   if (!bus_expire_list_add (list, &item->item))
-    _dbus_assert_not_reached ("out of memory");
+    _dbus_test_fatal ("out of memory");
 
   next_interval =
     do_expiration_with_monotonic_time (list, tv_sec_not_expired,
