@@ -320,6 +320,7 @@ handle_reload_watch (DBusWatch    *watch,
         loop = bus_context_get_loop (context);
         if (loop != NULL)
           {
+            _dbus_daemon_report_stopping ();
             _dbus_loop_quit (loop);
           }
       }
@@ -692,7 +693,6 @@ main (int argc, char **argv)
                              _dbus_string_get_length(&address) > 0 ? &address : NULL,
                              &error);
   _dbus_string_free (&config_file);
-  _dbus_string_free (&address);
   if (context == NULL)
     {
       _dbus_warn ("Failed to start message bus: %s",
@@ -717,6 +717,7 @@ main (int argc, char **argv)
 #endif /* DBUS_UNIX */
 
   _dbus_verbose ("We are on D-Bus...\n");
+  _dbus_daemon_report_ready ();
   _dbus_loop_run (bus_context_get_loop (context));
 
   bus_context_shutdown (context);
