@@ -28,34 +28,12 @@
 #include <dbus/dbus-string.h>
 #include <dbus/dbus-sysdeps.h>
 #include <dbus/dbus-internals.h>
+#include <dbus/dbus-test-tap.h>
 
 #if !defined(DBUS_ENABLE_EMBEDDED_TESTS) || !defined(DBUS_UNIX)
 #error This file is only relevant for the embedded tests on Unix
 #endif
 
-static void die (const char *failure) _DBUS_GNUC_NORETURN;
-
-static void
-die (const char *failure)
-{
-  fprintf (stderr, "Unit test failed: %s\n", failure);
-  exit (1);
-}
-
-static void
-check_memleaks (const char *name)
-{
-  dbus_shutdown ();
-
-  printf ("%s: checking for memleaks\n", name);
-  if (_dbus_get_malloc_blocks_outstanding () != 0)
-    {
-      _dbus_warn ("%d dbus_malloc blocks were not freed",
-                  _dbus_get_malloc_blocks_outstanding ());
-      die ("memleaks");
-    }
-}
-
 static void
 test_pre_hook (void)
 {
@@ -65,7 +43,7 @@ static const char *progname = "";
 static void
 test_post_hook (void)
 {
-  check_memleaks (progname);
+  _dbus_test_check_memleaks (progname);
 }
 
 int
@@ -82,23 +60,21 @@ main (int argc, char **argv)
     dir = _dbus_getenv ("DBUS_TEST_DATA");
 
   if (dir == NULL)
-    {
-      fprintf (stderr, "Must specify test data directory as argv[1] or in DBUS_TEST_DATA env variable\n");
-      return 1;
-    }
+    _dbus_test_fatal ("Must specify test data directory as argv[1] or in DBUS_TEST_DATA env variable");
 
   _dbus_string_init_const (&test_data_dir, dir);
 
   if (!_dbus_threads_init_debug ())
-    die ("initializing debug threads");
+    _dbus_test_fatal ("OOM initializing debug threads");
 
   test_pre_hook ();
-  printf ("%s: Running config file parser (trivial) test\n", argv[0]);
+  _dbus_test_diag ("%s: Running config file parser (trivial) test", argv[0]);
   if (!bus_config_parser_trivial_test (&test_data_dir))
-    die ("parser");
+    _dbus_test_fatal ("OOM creating parser");
+
+  /* All failure modes for this test are currently fatal */
+  _dbus_test_ok ("%s", argv[0]);
   test_post_hook ();
 
-  printf ("%s: Success\n", argv[0]);
-
-  return 0;
+  return _dbus_test_done_testing ();
 }
