@@ -529,7 +529,6 @@ sha1_handle_first_client_response (DBusAuth         *auth,
   DBusString tmp2;
   dbus_bool_t retval = FALSE;
   DBusError error = DBUS_ERROR_INIT;
-  DBusCredentials *myself = NULL;
 
   _dbus_string_set_length (&auth->challenge, 0);
   
@@ -550,10 +549,19 @@ sha1_handle_first_client_response (DBusAuth         *auth,
         }
     }
       
-  if (!_dbus_credentials_add_from_user (auth->desired_identity, data))
+  if (!_dbus_credentials_add_from_user (auth->desired_identity, data,
+                                        DBUS_CREDENTIALS_ADD_FLAGS_USER_DATABASE,
+                                        &error))
     {
-      _dbus_verbose ("%s: Did not get a valid username from client\n",
-                     DBUS_AUTH_NAME (auth));
+      if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY))
+        {
+          dbus_error_free (&error);
+          goto out;
+        }
+
+      _dbus_verbose ("%s: Did not get a valid username from client: %s\n",
+                     DBUS_AUTH_NAME (auth), error.message);
+      dbus_error_free (&error);
       return send_rejected (auth);
     }
       
@@ -566,34 +574,6 @@ sha1_handle_first_client_response (DBusAuth         *auth,
       return FALSE;
     }
 
-  myself = _dbus_credentials_new_from_current_process ();
-
-  if (myself == NULL)
-    goto out;
-
-  if (!_dbus_credentials_same_user (myself, auth->desired_identity))
-    {
-      /*
-       * DBUS_COOKIE_SHA1 is not suitable for authenticating that the
-       * client is anyone other than the user owning the process
-       * containing the DBusServer: we probably aren't allowed to write
-       * to other users' home directories. Even if we can (for example
-       * uid 0 on traditional Unix or CAP_DAC_OVERRIDE on Linux), we
-       * must not, because the other user controls their home directory,
-       * and could carry out symlink attacks to make us read from or
-       * write to unintended locations. It's difficult to avoid symlink
-       * attacks in a portable way, so we just don't try. This isn't a
-       * regression, because DBUS_COOKIE_SHA1 never worked for other
-       * users anyway.
-       */
-      _dbus_verbose ("%s: client tried to authenticate as \"%s\", "
-                     "but that doesn't match this process",
-                     DBUS_AUTH_NAME (auth),
-                     _dbus_string_get_const_data (data));
-      retval = send_rejected (auth);
-      goto out;
-    }
-
   /* we cache the keyring for speed, so here we drop it if it's the
    * wrong one. FIXME caching the keyring here is useless since we use
    * a different DBusAuth for every connection.
@@ -708,9 +688,6 @@ sha1_handle_first_client_response (DBusAuth         *auth,
   _dbus_string_zero (&tmp2);
   _dbus_string_free (&tmp2);
 
-  if (myself != NULL)
-    _dbus_credentials_unref (myself);
-
   return retval;
 }
 
@@ -1140,11 +1117,22 @@ handle_server_data_external_mech (DBusAuth         *auth,
     }
   else
     {
+      DBusError error = DBUS_ERROR_INIT;
+
       if (!_dbus_credentials_add_from_user (auth->desired_identity,
-                                            &auth->identity))
+                                            &auth->identity,
+                                            DBUS_CREDENTIALS_ADD_FLAGS_NONE,
+                                            &error))
         {
-          _dbus_verbose ("%s: could not get credentials from uid string\n",
-                         DBUS_AUTH_NAME (auth));
+          if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY))
+            {
+              dbus_error_free (&error);
+              return FALSE;
+            }
+
+          _dbus_verbose ("%s: could not get credentials from uid string: %s\n",
+                         DBUS_AUTH_NAME (auth), error.message);
+          dbus_error_free (&error);
           return send_rejected (auth);
         }
     }
@@ -1177,6 +1165,11 @@ handle_server_data_external_mech (DBusAuth         *auth,
                                              auth->credentials))
         return FALSE;
 
+      if (!_dbus_credentials_add_credential (auth->authorized_identity,
+                                             DBUS_CREDENTIAL_UNIX_GROUP_IDS,
+                                             auth->credentials))
+        return FALSE;
+
       if (!_dbus_credentials_add_credential (auth->authorized_identity,
                                              DBUS_CREDENTIAL_LINUX_SECURITY_LABEL,
                                              auth->credentials))
