@@ -24,6 +24,7 @@
 #include <config.h>
 #include "dbus-dataslot.h"
 #include "dbus-threads-internal.h"
+#include <dbus/dbus-test-tap.h>
 
 /**
  * @defgroup DBusDataSlot Data slots
@@ -376,7 +377,7 @@ _dbus_data_slot_test (void)
   void *old_data;
 
   if (!_dbus_data_slot_allocator_init (&allocator, _DBUS_LOCK_server_slots))
-    _dbus_assert_not_reached ("no memory for allocator");
+    _dbus_test_fatal ("no memory for allocator");
 
   _dbus_data_slot_list_init (&list);
 
@@ -394,7 +395,7 @@ _dbus_data_slot_test (void)
       _dbus_data_slot_allocator_alloc (&allocator, &tmp);
 
       if (tmp != i)
-        _dbus_assert_not_reached ("did not allocate slots in numeric order");
+        _dbus_test_fatal ("did not allocate slots in numeric order");
 
       ++i;
     }
@@ -407,7 +408,7 @@ _dbus_data_slot_test (void)
                                      _DBUS_INT_TO_POINTER (i), 
                                      test_free_slot_data_func,
                                      &old_free_func, &old_data))
-        _dbus_assert_not_reached ("no memory to set data");
+        _dbus_test_fatal ("no memory to set data");
 
       _dbus_assert (old_free_func == NULL);
       _dbus_assert (old_data == NULL);
@@ -427,7 +428,7 @@ _dbus_data_slot_test (void)
                                      _DBUS_INT_TO_POINTER (i), 
                                      test_free_slot_data_func,
                                      &old_free_func, &old_data))
-        _dbus_assert_not_reached ("no memory to set data");
+        _dbus_test_fatal ("no memory to set data");
 
       _dbus_assert (old_free_func == test_free_slot_data_func);
       _dbus_assert (_DBUS_POINTER_TO_INT (old_data) == i);
