@@ -26,6 +26,7 @@
 #include "dbus-protocol.h"
 #include "dbus-marshal-basic.h"
 #include "dbus-test.h"
+#include "dbus-test-tap.h"
 #include "dbus-valgrind-internal.h"
 #include <stdio.h>
 #include <stdarg.h>
@@ -446,7 +447,7 @@ _dbus_verbose_real (
 
 /**
  * Reinitializes the verbose logging code, used
- * as a hack in dbus-spawn.c so that a child
+ * as a hack in dbus-spawn-unix.c so that a child
  * process re-reads its pid
  *
  */
@@ -1006,7 +1007,7 @@ run_failing_each_malloc (int                    n_mallocs,
                      description, n_mallocs,
                      _dbus_get_fail_alloc_failures ());
 
-      if (!(* func) (data))
+      if (!(* func) (data, FALSE))
         return FALSE;
       
       n_mallocs -= 1;
@@ -1044,17 +1045,18 @@ _dbus_test_oom_handling (const char             *description,
   
   _dbus_set_fail_alloc_counter (_DBUS_INT_MAX);
 
-  _dbus_verbose ("Running once to count mallocs\n");
-  
-  if (!(* func) (data))
+  _dbus_test_diag ("Running \"%s\" once to count mallocs", description);
+
+  if (!(* func) (data, TRUE))
     return FALSE;
-  
+
   approx_mallocs = _DBUS_INT_MAX - _dbus_get_fail_alloc_counter ();
 
-  _dbus_verbose ("\n=================\n%s: about %d mallocs total\n=================\n",
-                 description, approx_mallocs);
+  _dbus_test_diag ("\"%s\" has about %d mallocs in total",
+                   description, approx_mallocs);
 
   setting = _dbus_getenv ("DBUS_TEST_MALLOC_FAILURES");
+
   if (setting != NULL)
     {
       DBusString str;
@@ -1070,23 +1072,36 @@ _dbus_test_oom_handling (const char             *description,
       max_failures_to_try = 4;
     }
 
+  if (RUNNING_ON_VALGRIND && _dbus_getenv ("DBUS_TEST_SLOW") == NULL)
+    {
+      /* The full OOM testing is slow, valgrind is slow, so the
+       * combination is just horrible. Only do this if the user
+       * asked for extra-slow tests. */
+      max_failures_to_try = 0;
+    }
+
   if (max_failures_to_try < 1)
     {
-      _dbus_verbose ("not testing OOM handling\n");
+      _dbus_test_diag ("not testing OOM handling");
       return TRUE;
     }
 
+  _dbus_test_diag ("testing \"%s\" with up to %d consecutive malloc failures",
+                   description, max_failures_to_try);
+
   i = setting ? max_failures_to_try - 1 : 1;
   while (i < max_failures_to_try)
     {
+      _dbus_test_diag ("testing \"%s\" with %d consecutive malloc failures",
+                       description, i + 1);
+
       _dbus_set_fail_alloc_failures (i);
       if (!run_failing_each_malloc (approx_mallocs, description, func, data))
         return FALSE;
       ++i;
     }
   
-  _dbus_verbose ("\n=================\n%s: all iterations passed\n=================\n",
-                 description);
+  _dbus_verbose ("\"%s\" coped OK with malloc failures", description);
 
   return TRUE;
 }
