@@ -192,13 +192,8 @@ void _dbus_real_assert_not_reached (const char *explanation,
 
 #define _DBUS_ZERO(object) (memset (&(object), '\0', sizeof ((object))))
 
-#ifdef offsetof
-#define _DBUS_STRUCT_OFFSET(struct_type, member) \
-    (offsetof (struct_type, member))
-#else
 #define _DBUS_STRUCT_OFFSET(struct_type, member)	\
     ((intptr_t) ((unsigned char*) &((struct_type*) 0)->member))
-#endif
 
 #define _DBUS_ALIGNOF(type) \
     (_DBUS_STRUCT_OFFSET (struct { char _1; type _2; }, _2))
@@ -306,7 +301,8 @@ void _dbus_set_error_valist (DBusError  *error,
                              const char *format,
                              va_list     args) _DBUS_GNUC_PRINTF (3, 0);
 
-typedef dbus_bool_t (* DBusTestMemoryFunction)  (void *data);
+typedef dbus_bool_t (* DBusTestMemoryFunction)  (void        *data,
+                                                 dbus_bool_t  have_memory);
 
 #ifdef DBUS_ENABLE_EMBEDDED_TESTS
 /* Memory debugging */
@@ -334,7 +330,7 @@ dbus_bool_t _dbus_test_oom_handling (const char             *description,
 #define _dbus_disable_mem_pools()            (FALSE)
 #define _dbus_get_malloc_blocks_outstanding() (0)
 
-#define _dbus_test_oom_handling(description, func, data) ((*func) (data))
+#define _dbus_test_oom_handling(description, func, data) ((*func) (data, TRUE))
 #endif /* !DBUS_ENABLE_EMBEDDED_TESTS */
 
 typedef void (* DBusShutdownFunction) (void *data);
@@ -370,7 +366,8 @@ typedef enum
   _DBUS_N_GLOBAL_LOCKS
 } DBusGlobalLock;
 
-dbus_bool_t _dbus_lock   (DBusGlobalLock lock) _DBUS_GNUC_WARN_UNUSED_RESULT;
+_DBUS_WARN_UNUSED_RESULT
+dbus_bool_t _dbus_lock   (DBusGlobalLock lock);
 void        _dbus_unlock (DBusGlobalLock lock);
 
 #define _DBUS_LOCK_NAME(name)           _DBUS_LOCK_##name
@@ -380,6 +377,7 @@ void        _dbus_unlock (DBusGlobalLock lock);
 DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_threads_init_debug (void);
 
+DBUS_PRIVATE_EXPORT
 dbus_bool_t   _dbus_address_append_escaped (DBusString       *escaped,
                                             const DBusString *unescaped);
 
@@ -402,7 +400,7 @@ union DBusGUID
   char as_bytes[DBUS_UUID_LENGTH_BYTES];                /**< guid as 16 single-byte values */
 };
 
-DBUS_PRIVATE_EXPORT _DBUS_GNUC_WARN_UNUSED_RESULT
+DBUS_PRIVATE_EXPORT _DBUS_WARN_UNUSED_RESULT
 dbus_bool_t _dbus_generate_uuid  (DBusGUID         *uuid,
                                   DBusError        *error);
 DBUS_PRIVATE_EXPORT
