@@ -26,6 +26,7 @@
 #ifdef DBUS_ENABLE_EMBEDDED_TESTS
 #include "dbus-marshal-byteswap.h"
 #include "dbus-test.h"
+#include <dbus/dbus-test-tap.h>
 #include <stdio.h>
 
 static void
@@ -37,24 +38,22 @@ do_byteswap_test (int byte_order)
   int opposite_order;
 
   if (!_dbus_string_init (&signature) || !_dbus_string_init (&body))
-    _dbus_assert_not_reached ("oom");
+    _dbus_test_fatal ("oom");
 
   opposite_order = byte_order == DBUS_LITTLE_ENDIAN ? DBUS_BIG_ENDIAN : DBUS_LITTLE_ENDIAN;
   
   sequence = 0;
-  while (dbus_internal_do_not_use_generate_bodies (sequence,
-                                                   byte_order,
-                                                   &signature, &body))
+  while (_dbus_test_generate_bodies (sequence, byte_order, &signature, &body))
     {
       DBusString copy;
       DBusTypeReader body_reader;
       DBusTypeReader copy_reader;
 
       if (!_dbus_string_init (&copy))
-        _dbus_assert_not_reached ("oom");
+        _dbus_test_fatal ("oom");
 
       if (!_dbus_string_copy (&body, 0, &copy, 0))
-        _dbus_assert_not_reached ("oom");
+        _dbus_test_fatal ("oom");
 
       _dbus_marshal_byteswap (&signature, 0,
                               byte_order,
@@ -75,8 +74,7 @@ do_byteswap_test (int byte_order)
           _dbus_verbose_bytes_of_string (&copy, 0,
                                          _dbus_string_get_length (&copy));
 
-          _dbus_warn ("Byte-swapped data did not have same values as original data");
-          _dbus_assert_not_reached ("test failed");
+          _dbus_test_fatal ("Byte-swapped data did not have same values as original data");
         }
       
       _dbus_string_free (&copy);
@@ -89,7 +87,7 @@ do_byteswap_test (int byte_order)
   _dbus_string_free (&signature);
   _dbus_string_free (&body);
 
-  printf ("  %d blocks swapped from order '%c' to '%c'\n",
+  _dbus_test_diag ("  %d blocks swapped from order '%c' to '%c'",
           sequence, byte_order, opposite_order);
 }
 
