@@ -29,6 +29,7 @@
 #include "dbus-marshal-basic.h"
 #include "dbus-signature.h"
 #include "dbus-internals.h"
+#include <dbus/dbus-test-tap.h>
 #include <string.h>
 
 #if !defined(PRIx64) && defined(DBUS_WIN)
@@ -226,7 +227,7 @@ data_block_verify (DataBlock *block)
       _dbus_verbose_bytes_of_string (&block->signature,
                                      offset,
                                      _dbus_string_get_length (&block->signature) - offset);
-      _dbus_assert_not_reached ("block did not verify: bad bytes at end of signature");
+      _dbus_test_fatal ("block did not verify: bad bytes at end of signature");
     }
   if (!_dbus_string_ends_with_c_str (&block->body,
                                      FENCE_BYTES_STR))
@@ -240,7 +241,7 @@ data_block_verify (DataBlock *block)
       _dbus_verbose_bytes_of_string (&block->body,
                                      offset,
                                      _dbus_string_get_length (&block->body) - offset);
-      _dbus_assert_not_reached ("block did not verify: bad bytes at end of body");
+      _dbus_test_fatal ("block did not verify: bad bytes at end of body");
     }
 
   _dbus_assert (_dbus_string_validate_nul (&block->signature,
@@ -307,12 +308,10 @@ real_check_expected_type (DBusTypeReader *reader,
 
   if (t != expected)
     {
-      _dbus_warn ("Read type %s while expecting %s at %s line %d",
+      _dbus_test_fatal ("Read wrong type: read type %s while expecting %s at %s line %d",
                   _dbus_type_to_string (t),
                   _dbus_type_to_string (expected),
                   funcname, line);
-
-      _dbus_assert_not_reached ("read wrong type");
     }
 }
 
@@ -320,17 +319,15 @@ real_check_expected_type (DBusTypeReader *reader,
 
 #define NEXT_EXPECTING_TRUE(reader)  do { if (!_dbus_type_reader_next (reader))         \
  {                                                                                      \
-    _dbus_warn ("_dbus_type_reader_next() should have returned TRUE at %s %d",          \
+    _dbus_test_fatal ("_dbus_type_reader_next() should have returned TRUE at %s %d",    \
                               _DBUS_FUNCTION_NAME, __LINE__);                           \
-    _dbus_assert_not_reached ("test failed");                                           \
  }                                                                                      \
 } while (0)
 
 #define NEXT_EXPECTING_FALSE(reader) do { if (_dbus_type_reader_next (reader))          \
  {                                                                                      \
-    _dbus_warn ("_dbus_type_reader_next() should have returned FALSE at %s %d",         \
+    _dbus_test_fatal ("_dbus_type_reader_next() should have returned FALSE at %s %d",   \
                               _DBUS_FUNCTION_NAME, __LINE__);                           \
-    _dbus_assert_not_reached ("test failed");                                           \
  }                                                                                      \
  check_expected_type (reader, DBUS_TYPE_INVALID);                                       \
 } while (0)
@@ -1025,7 +1022,7 @@ node_append_child (TestTypeNode *node,
   _dbus_assert (node->klass->instance_size >= (int) sizeof (TestTypeNodeContainer));
 
   if (!_dbus_list_append (&container->children, child))
-    _dbus_assert_not_reached ("no memory"); /* we never check the return value on node_append_child anyhow - it's run from outside the malloc-failure test code */
+    _dbus_test_fatal ("no memory"); /* we never check the return value on node_append_child anyhow - it's run from outside the malloc-failure test code */
 
   return TRUE;
 }
@@ -1117,7 +1114,7 @@ run_test_copy (NodeIterationData *nid)
       _dbus_verbose ("DEST\n");
       _dbus_verbose_bytes_of_string (&dest.signature, 0,
                                      _dbus_string_get_length (&dest.signature));
-      _dbus_assert_not_reached ("signatures did not match");
+      _dbus_test_fatal ("signatures did not match");
     }
 
   if (!_dbus_string_equal (&src->body, &dest.body))
@@ -1128,7 +1125,7 @@ run_test_copy (NodeIterationData *nid)
       _dbus_verbose ("DEST\n");
       _dbus_verbose_bytes_of_string (&dest.body, 0,
                                      _dbus_string_get_length (&dest.body));
-      _dbus_assert_not_reached ("bodies did not match");
+      _dbus_test_fatal ("bodies did not match");
     }
 
   retval = TRUE;
@@ -1323,7 +1320,7 @@ run_test_delete_values (NodeIterationData *nid)
               while (elem > 0)
                 {
                   if (!_dbus_type_reader_next (&array))
-                    _dbus_assert_not_reached ("should have had another element");
+                    _dbus_test_fatal ("should have had another element");
                   --elem;
                 }
 
@@ -1360,7 +1357,8 @@ run_test_delete_values (NodeIterationData *nid)
 }
 
 static dbus_bool_t
-run_test_nodes_iteration (void *data)
+run_test_nodes_iteration (void        *data,
+                          dbus_bool_t  have_memory)
 {
   NodeIterationData *nid = data;
   DBusTypeReader reader;
@@ -1398,11 +1396,10 @@ run_test_nodes_iteration (void *data)
   if (!_dbus_string_equal_substring (nid->signature, 0, _dbus_string_get_length (nid->signature),
                                      &nid->block->signature, nid->type_offset))
     {
-      _dbus_warn ("Expected signature '%s' and got '%s' with initial offset %d",
+      _dbus_test_fatal ("Expected signature '%s' and got '%s' with initial offset %d",
                   _dbus_string_get_const_data (nid->signature),
                   _dbus_string_get_const_data_len (&nid->block->signature, nid->type_offset, 0),
                   nid->type_offset);
-      _dbus_assert_not_reached ("wrong signature");
     }
 
   i = 0;
@@ -1465,7 +1462,7 @@ run_test_nodes_in_one_configuration (TestTypeNode    **nodes,
   NodeIterationData nid;
 
   if (!data_block_init (&block, byte_order, initial_offset))
-    _dbus_assert_not_reached ("no memory");
+    _dbus_test_fatal ("no memory");
 
   nid.signature = signature;
   nid.block = &block;
@@ -1482,8 +1479,8 @@ run_test_nodes_in_one_configuration (TestTypeNode    **nodes,
     }
   else
     {
-      if (!run_test_nodes_iteration (&nid))
-        _dbus_assert_not_reached ("no memory");
+      if (!run_test_nodes_iteration (&nid, TRUE))
+        _dbus_test_fatal ("no memory");
     }
 
   data_block_free (&block);
@@ -1497,13 +1494,13 @@ run_test_nodes (TestTypeNode **nodes,
   DBusString signature;
 
   if (!_dbus_string_init (&signature))
-    _dbus_assert_not_reached ("no memory");
+    _dbus_test_fatal ("no memory");
 
   i = 0;
   while (i < n_nodes)
     {
       if (! node_build_signature (nodes[i], &signature))
-        _dbus_assert_not_reached ("no memory");
+        _dbus_test_fatal ("no memory");
 
       ++i;
     }
@@ -1604,14 +1601,14 @@ build_body (TestTypeNode **nodes,
   while (i < n_nodes)
     {
       if (! node_build_signature (nodes[i], signature))
-        _dbus_assert_not_reached ("no memory");
-      
+        _dbus_test_fatal ("no memory");
+
       ++i;
     }
 
   if (!data_block_init (&block, byte_order, 0))
-    _dbus_assert_not_reached ("no memory");
-  
+    _dbus_test_fatal ("no memory");
+
   data_block_init_reader_writer (&block,
                                  &reader, &writer);
   
@@ -1620,13 +1617,13 @@ build_body (TestTypeNode **nodes,
    */
   if (!_dbus_string_insert_byte (&block.signature,
                                  0, '\0'))
-    _dbus_assert_not_reached ("no memory");
+    _dbus_test_fatal ("no memory");
 
   i = 0;
   while (i < n_nodes)
     {
       if (!node_write_value (nodes[i], &block, &writer, i))
-        _dbus_assert_not_reached ("no memory");
+        _dbus_test_fatal ("no memory");
 
       ++i;
     }
@@ -1634,16 +1631,16 @@ build_body (TestTypeNode **nodes,
   if (!_dbus_string_copy_len (&block.body, 0,
                               _dbus_string_get_length (&block.body) - N_FENCE_BYTES,
                               body, 0))
-    _dbus_assert_not_reached ("oom");
+    _dbus_test_fatal ("oom");
 
   data_block_free (&block);  
 }
 
 dbus_bool_t
-dbus_internal_do_not_use_generate_bodies (int           sequence,
-                                          int           byte_order,
-                                          DBusString   *signature,
-                                          DBusString   *body)
+_dbus_test_generate_bodies (int           sequence,
+                            int           byte_order,
+                            DBusString   *signature,
+                            DBusString   *body)
 {
   TestTypeNode *nodes[1];
   int i;
@@ -2070,27 +2067,15 @@ int16_from_seed (int seed)
    * just use seed itself, but that would only ever touch one byte of
    * the int so would miss some kinds of bug.
    */
-  dbus_int16_t v;
+  static const dbus_int16_t v_of_seed[5] = {
+    SAMPLE_INT16,
+    SAMPLE_INT16_ALTERNATE,
+    -1,
+    _DBUS_INT16_MAX,
+    1
+  };
 
-  v = 42; /* just to quiet compiler afaik */
-  switch (seed % 5)
-    {
-    case 0:
-      v = SAMPLE_INT16;
-      break;
-    case 1:
-      v = SAMPLE_INT16_ALTERNATE;
-      break;
-    case 2:
-      v = -1;
-      break;
-    case 3:
-      v = _DBUS_INT16_MAX;
-      break;
-    case 4:
-      v = 1;
-      break;
-    }
+  dbus_int16_t v = v_of_seed[seed % _DBUS_N_ELEMENTS(v_of_seed)];
 
   if (seed > 1)
     v *= seed; /* wraps around eventually, which is fine */
@@ -2207,27 +2192,15 @@ int32_from_seed (int seed)
    * just use seed itself, but that would only ever touch one byte of
    * the int so would miss some kinds of bug.
    */
-  dbus_int32_t v;
+  static const dbus_int32_t v_of_seed[5] = {
+    SAMPLE_INT32,
+    SAMPLE_INT32_ALTERNATE,
+    -1,
+    _DBUS_INT_MAX,
+    1
+  };
 
-  v = 42; /* just to quiet compiler afaik */
-  switch (seed % 5)
-    {
-    case 0:
-      v = SAMPLE_INT32;
-      break;
-    case 1:
-      v = SAMPLE_INT32_ALTERNATE;
-      break;
-    case 2:
-      v = -1;
-      break;
-    case 3:
-      v = _DBUS_INT_MAX;
-      break;
-    case 4:
-      v = 1;
-      break;
-    }
+  dbus_int32_t v = v_of_seed[seed % _DBUS_N_ELEMENTS(v_of_seed)];
 
   if (seed > 1)
     v *= seed; /* wraps around eventually, which is fine */
@@ -2414,6 +2387,9 @@ string_from_seed (char *buf,
    */
   switch (seed % 3)
     {
+    default:
+      /* don't alter it */
+      break;
     case 1:
       len += 2;
       break;
@@ -2477,11 +2453,7 @@ string_read_value (TestTypeNode   *node,
                     seed);
 
   if (strcmp (buf, v) != 0)
-    {
-      _dbus_warn ("read string '%s' expected '%s'",
-                  v, buf);
-      _dbus_assert_not_reached ("test failed");
-    }
+    _dbus_test_fatal ("read string '%s' expected '%s'", v, buf);
 
   return TRUE;
 }
@@ -2648,13 +2620,10 @@ double_read_value (TestTypeNode   *node,
   expected = double_from_seed (seed);
 
   if (!_DBUS_DOUBLES_BITWISE_EQUAL (v, expected))
-    {
-      _dbus_warn ("Expected double %g got %g\n bits = 0x%" PRIx64 " vs.\n bits = 0x%" PRIx64,
-                  expected, v,
-                  *(dbus_uint64_t*)(char*)&expected,
-                  *(dbus_uint64_t*)(char*)&v);
-      _dbus_assert_not_reached ("test failed");
-    }
+    _dbus_test_fatal ("Expected double %g got %g\n bits = 0x%" PRIx64 " vs.\n bits = 0x%" PRIx64,
+                      expected, v,
+                      *(dbus_uint64_t*)(char*)&expected,
+                      *(dbus_uint64_t*)(char*)&v);
 
   return TRUE;
 }
@@ -2745,11 +2714,7 @@ object_path_read_value (TestTypeNode   *node,
   object_path_from_seed (buf, seed);
 
   if (strcmp (buf, v) != 0)
-    {
-      _dbus_warn ("read object path '%s' expected '%s'",
-                  v, buf);
-      _dbus_assert_not_reached ("test failed");
-    }
+    _dbus_test_fatal ("read object path '%s' expected '%s'", v, buf);
 
   return TRUE;
 }
@@ -2820,11 +2785,7 @@ signature_read_value (TestTypeNode   *node,
   signature_from_seed (buf, seed);
 
   if (strcmp (buf, v) != 0)
-    {
-      _dbus_warn ("read signature value '%s' expected '%s'",
-                  v, buf);
-      _dbus_assert_not_reached ("test failed");
-    }
+    _dbus_test_fatal ("read signature value '%s' expected '%s'", v, buf);
 
   return TRUE;
 }
