@@ -156,7 +156,7 @@ _dbus_string_init_preallocated (DBusString *str,
   
   real->constant = FALSE;
   real->locked = FALSE;
-  real->invalid = FALSE;
+  real->valid = TRUE;
   real->align_offset = 0;
   
   fixup_alignment (real);
@@ -225,7 +225,7 @@ _dbus_string_init_const_len (DBusString *str,
   real->allocated = real->len + _DBUS_STRING_ALLOCATION_PADDING; /* a lie, just to avoid special-case assertions... */
   real->constant = TRUE;
   real->locked = TRUE;
-  real->invalid = FALSE;
+  real->valid = TRUE;
   real->align_offset = 0;
 
   /* We don't require const strings to be 8-byte aligned as the
@@ -251,7 +251,12 @@ _dbus_string_init_from_string(DBusString       *str,
 }
 
 /**
- * Frees a string created by _dbus_string_init().
+ * Frees a string created by _dbus_string_init(), and fills it with the
+ * same contents as #_DBUS_STRING_INIT_INVALID.
+ *
+ * Unlike all other #DBusString API, it is also valid to call this function
+ * for a string that was filled with #_DBUS_STRING_INIT_INVALID.
+ * This is convenient for error rollback.
  *
  * @param str memory where the string is stored.
  */
@@ -259,21 +264,33 @@ void
 _dbus_string_free (DBusString *str)
 {
   DBusRealString *real = (DBusRealString*) str;
+  /* DBusRealString and DBusString have the same members in the same order,
+   * just differently-named */
+  DBusRealString invalid = _DBUS_STRING_INIT_INVALID;
+
+  /* Allow for the _DBUS_STRING_INIT_INVALID case */
+  if (real->str == NULL && real->len == 0 && real->allocated == 0 &&
+      !real->constant && !real->locked && !real->valid &&
+      real->align_offset == 0)
+    return;
+
   DBUS_GENERIC_STRING_PREAMBLE (real);
   
   if (real->constant)
-    return;
+    goto wipe;
 
   /* so it's safe if @p str returned by a failed
    * _dbus_string_init call
    * Bug: https://bugs.freedesktop.org/show_bug.cgi?id=65959
    */
   if (real->str == NULL)
-    return;
+    goto wipe;
 
   dbus_free (real->str - real->align_offset);
 
-  real->invalid = TRUE;
+wipe:
+  *real = invalid;
+  real->valid = FALSE;
 }
 
 static dbus_bool_t
@@ -1818,7 +1835,7 @@ _dbus_string_skip_blank (const DBusString *str,
       ++i;
     }
 
-  _dbus_assert (i == real->len || !DBUS_IS_ASCII_BLANK (real->str[i]));
+  _dbus_assert (i == real->len || !DBUS_IS_ASCII_WHITE (real->str[i]));
   
   if (end)
     *end = i;
