@@ -35,6 +35,7 @@
 #include "dbus-test.h"
 
 #include <sys/types.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <signal.h>
@@ -99,6 +100,12 @@ _dbus_become_daemon (const DBusString *pidfile,
     }
 
   _dbus_verbose ("forking...\n");
+
+  /* Make sure our output buffers aren't redundantly printed by both the
+   * parent and the child */
+  fflush (stdout);
+  fflush (stderr);
+
   switch ((child_pid = fork ()))
     {
     case -1:
@@ -406,15 +413,23 @@ _dbus_rlimit_save_fd_limit (DBusError *error)
   return self;
 }
 
-/* Enough fds that we shouldn't run out, even if several uids work
- * together to carry out a denial-of-service attack. This happens to be
- * the same number that systemd < 234 would normally use. */
-#define ENOUGH_FDS 65536
-
 dbus_bool_t
-_dbus_rlimit_raise_fd_limit (DBusError *error)
+_dbus_rlimit_raise_fd_limit_if_privileged (unsigned int  desired,
+                                           DBusError    *error)
 {
-  struct rlimit old, lim;
+  struct rlimit lim;
+
+  /* No point to doing this practically speaking
+   * if we're not uid 0.  We expect the system
+   * bus to use this before we change UID, and
+   * the session bus takes the Linux default,
+   * currently 1024 for cur and 4096 for max.
+   */
+  if (getuid () != 0)
+    {
+      /* not an error, we're probably the session bus */
+      return TRUE;
+    }
 
   if (getrlimit (RLIMIT_NOFILE, &lim) < 0)
     {
@@ -423,50 +438,22 @@ _dbus_rlimit_raise_fd_limit (DBusError *error)
       return FALSE;
     }
 
-  old = lim;
-
-  if (getuid () == 0)
+  if (lim.rlim_cur == RLIM_INFINITY || lim.rlim_cur >= desired)
     {
-      /* We are privileged, so raise the soft limit to at least
-       * ENOUGH_FDS, and the hard limit to at least the desired soft
-       * limit. This assumes we can exercise CAP_SYS_RESOURCE on Linux,
-       * or other OSs' equivalents. */
-      if (lim.rlim_cur != RLIM_INFINITY &&
-          lim.rlim_cur < ENOUGH_FDS)
-        lim.rlim_cur = ENOUGH_FDS;
-
-      if (lim.rlim_max != RLIM_INFINITY &&
-          lim.rlim_max < lim.rlim_cur)
-        lim.rlim_max = lim.rlim_cur;
+      /* not an error, everything is fine */
+      return TRUE;
     }
 
-  /* Raise the soft limit to match the hard limit, which we can do even
-   * if we are unprivileged. In particular, systemd >= 240 will normally
-   * set rlim_cur to 1024 and rlim_max to 512*1024, recent Debian
-   * versions end up setting rlim_cur to 1024 and rlim_max to 1024*1024,
-   * and older and non-systemd Linux systems would typically set rlim_cur
-   * to 1024 and rlim_max to 4096. */
-  if (lim.rlim_max == RLIM_INFINITY || lim.rlim_cur < lim.rlim_max)
-    {
-#if defined(__APPLE__) && defined(__MACH__)
-      /* macOS 10.5 and above no longer allows RLIM_INFINITY for rlim_cur */
-      lim.rlim_cur = MIN (OPEN_MAX, lim.rlim_max);
-#else
-      lim.rlim_cur = lim.rlim_max;
-#endif
-    }
-
-  /* Early-return if there is nothing to do. */
-  if (lim.rlim_max == old.rlim_max &&
-      lim.rlim_cur == old.rlim_cur)
-    return TRUE;
+  /* Ignore "maximum limit", assume we have the "superuser"
+   * privileges.  On Linux this is CAP_SYS_RESOURCE.
+   */
+  lim.rlim_cur = lim.rlim_max = desired;
 
   if (setrlimit (RLIMIT_NOFILE, &lim) < 0)
     {
       dbus_set_error (error, _dbus_error_from_errno (errno),
-                      "Failed to set fd limit to %lu: %s",
-                      (unsigned long) lim.rlim_cur,
-                      _dbus_strerror (errno));
+                      "Failed to set fd limit to %u: %s",
+                      desired, _dbus_strerror (errno));
       return FALSE;
     }
 
@@ -505,7 +492,8 @@ _dbus_rlimit_save_fd_limit (DBusError *error)
 }
 
 dbus_bool_t
-_dbus_rlimit_raise_fd_limit (DBusError *error)
+_dbus_rlimit_raise_fd_limit_if_privileged (unsigned int  desired,
+                                           DBusError    *error)
 {
   fd_limit_not_supported (error);
   return FALSE;
@@ -1532,157 +1520,51 @@ _dbus_get_session_config_file (DBusString *str)
   return _dbus_string_append (str, DBUS_SESSION_CONFIG_FILE);
 }
 
-#ifdef DBUS_ENABLE_EMBEDDED_TESTS
-
-/*
- * Set uid to a machine-readable authentication identity (numeric Unix
- * uid or ConvertSidToStringSid-style Windows SID) that is likely to exist,
- * and differs from the identity of the current process.
- *
- * @param uid Populated with a machine-readable authentication identity
- *    on success
- * @returns #FALSE if no memory
+/**
+ * Report to a service manager that the daemon calling this function is
+ * ready for use. This is currently only implemented for systemd.
  */
-dbus_bool_t
-_dbus_test_append_different_uid (DBusString *uid)
+void
+_dbus_daemon_report_ready (void)
 {
-  if (geteuid () == 0)
-    return _dbus_string_append (uid, "65534");
-  else
-    return _dbus_string_append (uid, "0");
-}
-
-/*
- * Set uid to a human-readable authentication identity (login name)
- * that is likely to exist, and differs from the identity of the current
- * process. This function currently only exists on Unix platforms.
- *
- * @param uid Populated with a machine-readable authentication identity
- *    on success
- * @returns #FALSE if no memory
- */
-dbus_bool_t
-_dbus_test_append_different_username (DBusString *username)
-{
-  if (geteuid () == 0)
-    return _dbus_string_append (username, "nobody");
-  else
-    return _dbus_string_append (username, "root");
-}
-
+#ifdef HAVE_SYSTEMD
+  sd_notify (0, "READY=1");
 #endif
+}
 
 /**
- * If the current process has been protected from the Linux OOM killer
- * (the oom_score_adj process parameter is negative), reset it to the
- * default level of protection from the OOM killer (set oom_score_adj
- * to zero).
- *
- * This function does not use DBusError, to avoid calling malloc(), so
- * that it can be used in contexts where an async-signal-safe function
- * is required (for example after fork()). Instead, on failure it sets
- * errno and returns something like "Failed to open /dev/null" in
- * *error_str_p. Callers are expected to combine *error_str_p
- * with _dbus_strerror (errno) to get a full error report.
+ * Report to a service manager that the daemon calling this function is
+ * reloading configuration. This is currently only implemented for systemd.
  */
-dbus_bool_t
-_dbus_reset_oom_score_adj (const char **error_str_p)
+void
+_dbus_daemon_report_reloading (void)
 {
-#ifdef __linux__
-  int fd = -1;
-  dbus_bool_t ret = FALSE;
-  int saved_errno = 0;
-  const char *error_str = NULL;
-
-#ifdef O_CLOEXEC
-  fd = open ("/proc/self/oom_score_adj", O_RDWR | O_CLOEXEC);
+#ifdef HAVE_SYSTEMD
+  sd_notify (0, "RELOADING=1");
+#endif
+}
+
+/**
+ * Report to a service manager that the daemon calling this function is
+ * reloading configuration. This is currently only implemented for systemd.
+ */
+void
+_dbus_daemon_report_reloaded (void)
+{
+#ifdef HAVE_SYSTEMD
+  /* For systemd, this is the same code */
+  _dbus_daemon_report_ready ();
 #endif
-
-  if (fd < 0)
-    {
-      fd = open ("/proc/self/oom_score_adj", O_RDWR);
-      _dbus_fd_set_close_on_exec (fd);
-    }
-
-  if (fd >= 0)
-    {
-      ssize_t read_result = -1;
-      /* It doesn't actually matter whether we read the whole file,
-       * as long as we get the presence or absence of the minus sign */
-      char first_char = '\0';
-
-      read_result = read (fd, &first_char, 1);
-
-      if (read_result < 0)
-        {
-          /* This probably can't actually happen in practice: if we can
-           * open it, then we can hopefully read from it */
-          ret = FALSE;
-          error_str = "failed to read from /proc/self/oom_score_adj";
-          saved_errno = errno;
-          goto out;
-        }
-
-      /* If we are running with protection from the OOM killer
-       * (typical for the system dbus-daemon under systemd), then
-       * oom_score_adj will be negative. Drop that protection,
-       * returning to oom_score_adj = 0.
-       *
-       * Conversely, if we are running with increased susceptibility
-       * to the OOM killer (as user sessions typically do in
-       * systemd >= 250), oom_score_adj will be strictly positive,
-       * and we are not allowed to decrease it to 0 without privileges.
-       *
-       * If it's exactly 0 (typical for non-systemd systems, and
-       * user processes on older systemd) then there's no need to
-       * alter it.
-       *
-       * We shouldn't get an empty result, but if we do, assume it
-       * means zero and don't try to change it. */
-      if (read_result == 0 || first_char != '-')
-        {
-          /* Nothing needs to be done: the OOM score adjustment is
-           * non-negative */
-          ret = TRUE;
-          goto out;
-        }
-
-      if (pwrite (fd, "0", sizeof (char), 0) < 0)
-        {
-          ret = FALSE;
-          error_str = "writing oom_score_adj error";
-          saved_errno = errno;
-          goto out;
-        }
-
-      /* Success */
-      ret = TRUE;
-    }
-  else if (errno == ENOENT)
-    {
-      /* If /proc/self/oom_score_adj doesn't exist, assume the kernel
-       * doesn't support this feature and ignore it. */
-      ret = TRUE;
-    }
-  else
-    {
-      ret = FALSE;
-      error_str = "open(/proc/self/oom_score_adj)";
-      saved_errno = errno;
-      goto out;
-    }
-
-out:
-  if (fd >= 0)
-    _dbus_close (fd, NULL);
-
-  if (error_str_p != NULL)
-    *error_str_p = error_str;
-
-  errno = saved_errno;
-  return ret;
-#else
-  /* nothing to do on this platform */
-  return TRUE;
+}
+
+/**
+ * Report to a service manager that the daemon calling this function is
+ * shutting down. This is currently only implemented for systemd.
+ */
+void
+_dbus_daemon_report_stopping (void)
+{
+#ifdef HAVE_SYSTEMD
+  sd_notify (0, "STOPPING=1");
 #endif
 }
