@@ -273,7 +273,8 @@ _dbus_rlimit_save_fd_limit (DBusError *error)
 }
 
 dbus_bool_t
-_dbus_rlimit_raise_fd_limit (DBusError *error)
+_dbus_rlimit_raise_fd_limit_if_privileged (unsigned int  desired,
+                                           DBusError    *error)
 {
   fd_limit_not_supported (error);
   return FALSE;
@@ -1654,27 +1655,22 @@ _dbus_get_session_config_file (DBusString *str)
   return _dbus_get_config_file_name(str, "session.conf");
 }
 
-#ifdef DBUS_ENABLE_EMBEDDED_TESTS
-
-#define ANONYMOUS_SID "S-1-5-7"
-#define LOCAL_SYSTEM_SID "S-1-5-18"
-
-dbus_bool_t
-_dbus_test_append_different_uid (DBusString *uid)
+void
+_dbus_daemon_report_ready (void)
 {
-  char *sid = NULL;
-  dbus_bool_t ret;
-
-  if (!_dbus_getsid (&sid, _dbus_getpid ()))
-    return FALSE;
-
-  if (strcmp (sid, ANONYMOUS_SID) == 0)
-    ret = _dbus_string_append (uid, LOCAL_SYSTEM_SID);
-  else
-    ret = _dbus_string_append (uid, ANONYMOUS_SID);
-
-  LocalFree (sid);
-  return ret;
 }
 
-#endif
+void
+_dbus_daemon_report_reloading (void)
+{
+}
+
+void
+_dbus_daemon_report_reloaded (void)
+{
+}
+
+void
+_dbus_daemon_report_stopping (void)
+{
+}
