@@ -48,32 +48,6 @@
 #include <string.h>
 #include <stdarg.h>
 
-#if !defined(BROKEN_POLL) && (defined(__APPLE__) || defined(__INTERIX))
-  /* Following libcurl's example, we blacklist poll() on Darwin
-   * (macOS, iOS, etc.) and Interix due to a history of implementation
-   * issues.
-   * https://github.com/curl/curl/blob/master/m4/curl-functions.m4
-   *
-   * On unspecified older macOS versions, poll() failed if given a
-   * device node to poll.
-   *
-   * On macOS < 10.9, poll() with nfds=0 failed instead of waiting for
-   * the timeout and then succeeding.
-   *
-   * On macOS >= 10.12, poll() with nfds=0 succeeded immediately
-   * instead of waiting for the timeout, resulting in busy-looping.
-   *
-   * On Interix, poll() apparently only works for files in /proc.
-   *
-   * The "legacy" build flavour in our CI machinery defines BROKEN_POLL
-   * on whatever platform is in use (normally Linux) to force use of the
-   * same select()-based poll() emulation that we use for macOS, Interix,
-   * and any platform that lacks a real poll(), so that we can test it
-   * more regularly.
-   */
-# define BROKEN_POLL
-#endif
-
 /* AIX sys/poll.h does #define events reqevents, and other
  * wonderousness, so must include sys/poll before declaring
  * DBusPollFD
@@ -158,15 +132,18 @@ typedef struct { SOCKET sock; } DBusSocket;
 # define DBUS_SOCKET_FORMAT "Iu"
 # define DBUS_SOCKET_INIT { INVALID_SOCKET }
 
+_DBUS_WARN_UNUSED_RESULT
 static inline SOCKET
 _dbus_socket_printable (DBusSocket s) { return s.sock; }
 
+_DBUS_WARN_UNUSED_RESULT
 static inline dbus_bool_t
 _dbus_socket_is_valid (DBusSocket s) { return s.sock != INVALID_SOCKET; }
 
 static inline void
 _dbus_socket_invalidate (DBusSocket *s) { s->sock = INVALID_SOCKET; }
 
+_DBUS_WARN_UNUSED_RESULT
 static inline int
 _dbus_socket_get_int (DBusSocket s) { return (int)s.sock; }
 
@@ -176,20 +153,24 @@ typedef struct { int fd; } DBusSocket;
 # define DBUS_SOCKET_FORMAT "d"
 # define DBUS_SOCKET_INIT { -1 }
 
+_DBUS_WARN_UNUSED_RESULT
 static inline int
 _dbus_socket_printable (DBusSocket s) { return s.fd; }
 
+_DBUS_WARN_UNUSED_RESULT
 static inline dbus_bool_t
 _dbus_socket_is_valid (DBusSocket s) { return s.fd >= 0; }
 
 static inline void
 _dbus_socket_invalidate (DBusSocket *s) { s->fd = -1; }
 
+_DBUS_WARN_UNUSED_RESULT
 static inline int
 _dbus_socket_get_int (DBusSocket s) { return s.fd; }
 
 #endif /* not DBUS_WIN */
 
+_DBUS_WARN_UNUSED_RESULT
 static inline DBusSocket
 _dbus_socket_get_invalid (void)
 {
@@ -256,6 +237,7 @@ int _dbus_listen_tcp_socket   (const char     *host,
                                const char     *port,
                                const char     *family,
                                DBusString     *retport,
+                               const char    **retfamily,
                                DBusSocket    **fds_p,
                                DBusError      *error);
 DBusSocket _dbus_accept       (DBusSocket      listen_fd);
@@ -266,8 +248,17 @@ dbus_bool_t _dbus_read_credentials_socket (DBusSocket        client_fd,
 dbus_bool_t _dbus_send_credentials_socket (DBusSocket       server_fd,
                                            DBusError       *error);
 
-dbus_bool_t _dbus_credentials_add_from_user            (DBusCredentials  *credentials,
-                                                        const DBusString *username);
+typedef enum
+{
+  DBUS_CREDENTIALS_ADD_FLAGS_USER_DATABASE = (1 << 0),
+  DBUS_CREDENTIALS_ADD_FLAGS_NONE = 0
+} DBusCredentialsAddFlags;
+
+dbus_bool_t _dbus_credentials_add_from_user            (DBusCredentials         *credentials,
+                                                        const DBusString        *username,
+                                                        DBusCredentialsAddFlags  flags,
+                                                        DBusError               *error);
+
 dbus_bool_t _dbus_credentials_add_from_current_process (DBusCredentials  *credentials);
 DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_append_user_from_current_process     (DBusString        *str);
@@ -479,7 +470,7 @@ const char* _dbus_get_tmpdir      (void);
 /**
  * Random numbers 
  */
-_DBUS_GNUC_WARN_UNUSED_RESULT
+_DBUS_WARN_UNUSED_RESULT
 dbus_bool_t _dbus_generate_random_bytes_buffer (char       *buffer,
                                                 int         n_bytes,
                                                 DBusError  *error);
@@ -496,6 +487,8 @@ const char* _dbus_error_from_errno (int error_number);
 DBUS_PRIVATE_EXPORT
 const char* _dbus_error_from_system_errno (void);
 
+int         _dbus_get_low_level_socket_errno         (void);
+
 int         _dbus_save_socket_errno                  (void);
 void        _dbus_restore_socket_errno               (int saved_errno);
 void        _dbus_set_errno_to_zero                  (void);
@@ -698,20 +691,34 @@ dbus_bool_t _dbus_replace_install_prefix (DBusString *path);
 typedef struct DBusRLimit DBusRLimit;
 
 DBusRLimit     *_dbus_rlimit_save_fd_limit                 (DBusError    *error);
-dbus_bool_t     _dbus_rlimit_raise_fd_limit                (DBusError    *error);
+dbus_bool_t     _dbus_rlimit_raise_fd_limit_if_privileged  (unsigned int  desired,
+                                                            DBusError    *error);
 dbus_bool_t     _dbus_rlimit_restore_fd_limit              (DBusRLimit   *saved,
                                                             DBusError    *error);
 void            _dbus_rlimit_free                          (DBusRLimit   *lim);
 
-#ifdef DBUS_ENABLE_EMBEDDED_TESTS
-_DBUS_GNUC_WARN_UNUSED_RESULT
-dbus_bool_t _dbus_test_append_different_uid (DBusString *uid);
+void            _dbus_daemon_report_ready                  (void);
+void            _dbus_daemon_report_reloading              (void);
+void            _dbus_daemon_report_reloaded               (void);
+void            _dbus_daemon_report_stopping               (void);
 
-#ifdef DBUS_UNIX
-_DBUS_GNUC_WARN_UNUSED_RESULT
-dbus_bool_t _dbus_test_append_different_username (DBusString *username);
-#endif
-#endif  /* DBUS_ENABLE_EMBEDDED_TESTS */
+dbus_bool_t _dbus_inet_sockaddr_to_string (const void *sockaddr_pointer,
+                                           size_t len,
+                                           char *string,
+                                           size_t string_len,
+                                           const char **family_name,
+                                           dbus_uint16_t *port,
+                                           DBusError *error);
+void _dbus_set_error_with_inet_sockaddr (DBusError *error,
+                                         const void *sockaddr_pointer,
+                                         size_t len,
+                                         const char *description,
+                                         int saved_errno);
+void _dbus_combine_tcp_errors (DBusList **sources,
+                               const char *summary,
+                               const char *host,
+                               const char *port,
+                               DBusError *dest);
 
 /** @} */
 
