@@ -30,62 +30,46 @@
 #include "dbus-credentials.h"
 #include <string.h>
 
+/* It isn't obvious from its name, but this file is part of the Unix
+ * system-dependent part of libdbus. Windows has a parallel
+ * implementation of some of it in dbus-sysdeps-win.c. */
+#if defined(DBUS_WIN) || !defined(DBUS_UNIX)
+#error "This file only makes sense on Unix OSs"
+#endif
+
 /**
  * @addtogroup DBusInternalsUtils
  * @{
  */
 
-static DBusUserInfo *
-_dbus_user_info_ref (DBusUserInfo *info)
-{
-  _dbus_assert (info->refcount > 0);
-  _dbus_assert (info->refcount < SIZE_MAX);
-  info->refcount++;
-  return info;
-}
-
 /**
- * Decrements the reference count. If it reaches 0,
- * frees the given #DBusUserInfo's members with _dbus_user_info_free()
+ * Frees the given #DBusUserInfo's members with _dbus_user_info_free()
  * and also calls dbus_free() on the block itself
  *
  * @param info the info
  */
 void
-_dbus_user_info_unref (DBusUserInfo *info)
+_dbus_user_info_free_allocated (DBusUserInfo *info)
 {
   if (info == NULL) /* hash table will pass NULL */
     return;
 
-  _dbus_assert (info->refcount > 0);
-  _dbus_assert (info->refcount < SIZE_MAX);
-
-  if (--info->refcount > 0)
-    return;
-
   _dbus_user_info_free (info);
   dbus_free (info);
 }
 
 /**
- * Decrements the reference count. If it reaches 0,
- * frees the given #DBusGroupInfo's members with _dbus_group_info_free()
+ * Frees the given #DBusGroupInfo's members with _dbus_group_info_free()
  * and also calls dbus_free() on the block itself
  *
  * @param info the info
  */
 void
-_dbus_group_info_unref (DBusGroupInfo *info)
+_dbus_group_info_free_allocated (DBusGroupInfo *info)
 {
   if (info == NULL) /* hash table will pass NULL */
     return;
 
-  _dbus_assert (info->refcount > 0);
-  _dbus_assert (info->refcount < SIZE_MAX);
-
-  if (--info->refcount > 0)
-    return;
-
   _dbus_group_info_free (info);
   dbus_free (info);
 }
@@ -145,9 +129,9 @@ _dbus_is_a_number (const DBusString *str,
  * @param uid the user ID or #DBUS_UID_UNSET
  * @param username username or #NULL 
  * @param error error to fill in
- * @returns the entry in the database (borrowed, do not free)
+ * @returns the entry in the database
  */
-const DBusUserInfo *
+DBusUserInfo*
 _dbus_user_database_lookup (DBusUserDatabase *db,
                             dbus_uid_t        uid,
                             const DBusString *username,
@@ -193,14 +177,13 @@ _dbus_user_database_lookup (DBusUserDatabase *db,
           dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
           return NULL;
         }
-      info->refcount = 1;
 
       if (uid != DBUS_UID_UNSET)
         {
           if (!_dbus_user_info_fill_uid (info, uid, error))
             {
               _DBUS_ASSERT_ERROR_IS_SET (error);
-              _dbus_user_info_unref (info);
+              _dbus_user_info_free_allocated (info);
               return NULL;
             }
         }
@@ -209,7 +192,7 @@ _dbus_user_database_lookup (DBusUserDatabase *db,
           if (!_dbus_user_info_fill (info, username, error))
             {
               _DBUS_ASSERT_ERROR_IS_SET (error);
-              _dbus_user_info_unref (info);
+              _dbus_user_info_free_allocated (info);
               return NULL;
             }
         }
@@ -219,35 +202,22 @@ _dbus_user_database_lookup (DBusUserDatabase *db,
       username = NULL;
 
       /* insert into hash */
-      if (_dbus_hash_table_insert_uintptr (db->users, info->uid, info))
-        {
-          _dbus_user_info_ref (info);
-        }
-      else
+      if (!_dbus_hash_table_insert_uintptr (db->users, info->uid, info))
         {
           dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
-          _dbus_user_info_unref (info);
+          _dbus_user_info_free_allocated (info);
           return NULL;
         }
 
-      if (_dbus_hash_table_insert_string (db->users_by_name,
-                                          info->username,
-                                          info))
-        {
-          _dbus_user_info_ref (info);
-        }
-      else
+      if (!_dbus_hash_table_insert_string (db->users_by_name,
+                                           info->username,
+                                           info))
         {
           _dbus_hash_table_remove_uintptr (db->users, info->uid);
           dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
-          _dbus_user_info_unref (info);
           return NULL;
         }
-
-      _dbus_user_info_unref (info);
-
-      /* Return a borrowed pointer to the DBusUserInfo owned by the
-       * hash tables */
+      
       return info;
     }
 }
@@ -554,25 +524,57 @@ _dbus_homedir_from_uid (dbus_uid_t         uid,
  * @returns #TRUE if the username existed and we got some credentials
  */
 dbus_bool_t
-_dbus_credentials_add_from_user (DBusCredentials  *credentials,
-                                 const DBusString *username)
+_dbus_credentials_add_from_user (DBusCredentials         *credentials,
+                                 const DBusString        *username,
+                                 DBusCredentialsAddFlags  flags,
+                                 DBusError               *error)
 {
   DBusUserDatabase *db;
   const DBusUserInfo *info;
+  unsigned long uid = DBUS_UID_UNSET;
+
+  /* Fast-path for the common case: if the "username" is all-numeric,
+   * then it's a Unix uid. This is true regardless of whether that uid
+   * exists in NSS or /etc/passwd or equivalent. */
+  if (_dbus_is_a_number (username, &uid))
+    {
+      _DBUS_STATIC_ASSERT (sizeof (uid) == sizeof (dbus_uid_t));
+
+      if (_dbus_credentials_add_unix_uid (credentials, uid))
+        {
+          return TRUE;
+        }
+      else
+        {
+          _DBUS_SET_OOM (error);
+          return FALSE;
+        }
+    }
+
+  /* If we aren't allowed to look in NSS or /etc/passwd, fail now. */
+  if (!(flags & DBUS_CREDENTIALS_ADD_FLAGS_USER_DATABASE))
+    {
+      dbus_set_error (error, DBUS_ERROR_INVALID_ARGS,
+                      "Expected a numeric Unix uid");
+      return FALSE;
+    }
 
-  /* FIXME: this can't distinguish ENOMEM from other errors */
   if (!_dbus_user_database_lock_system ())
-    return FALSE;
+    {
+      _DBUS_SET_OOM (error);
+      return FALSE;
+    }
 
   db = _dbus_user_database_get_system ();
   if (db == NULL)
     {
       _dbus_user_database_unlock_system ();
+      _DBUS_SET_OOM (error);
       return FALSE;
     }
 
   if (!_dbus_user_database_get_username (db, username,
-                                         &info, NULL))
+                                         &info, error))
     {
       _dbus_user_database_unlock_system ();
       return FALSE;
@@ -581,6 +583,7 @@ _dbus_credentials_add_from_user (DBusCredentials  *credentials,
   if (!_dbus_credentials_add_unix_uid(credentials, info->uid))
     {
       _dbus_user_database_unlock_system ();
+      _DBUS_SET_OOM (error);
       return FALSE;
     }
   
@@ -605,24 +608,24 @@ _dbus_user_database_new (void)
   db->refcount = 1;
 
   db->users = _dbus_hash_table_new (DBUS_HASH_UINTPTR,
-                                    NULL, (DBusFreeFunction) _dbus_user_info_unref);
+                                    NULL, (DBusFreeFunction) _dbus_user_info_free_allocated);
   
   if (db->users == NULL)
     goto failed;
 
   db->groups = _dbus_hash_table_new (DBUS_HASH_UINTPTR,
-                                     NULL, (DBusFreeFunction) _dbus_group_info_unref);
+                                     NULL, (DBusFreeFunction) _dbus_group_info_free_allocated);
   
   if (db->groups == NULL)
     goto failed;
 
   db->users_by_name = _dbus_hash_table_new (DBUS_HASH_STRING,
-                                            NULL, (DBusFreeFunction) _dbus_user_info_unref);
+                                            NULL, NULL);
   if (db->users_by_name == NULL)
     goto failed;
   
   db->groups_by_name = _dbus_hash_table_new (DBUS_HASH_STRING,
-                                             NULL, (DBusFreeFunction) _dbus_group_info_unref);
+                                             NULL, NULL);
   if (db->groups_by_name == NULL)
     goto failed;
   
