@@ -45,7 +45,7 @@ typedef dbus_bool_t (* DBusWatchHandler) (DBusWatch    *watch,
                                           void         *data);
 
 DBUS_PRIVATE_EXPORT
-DBusWatch* _dbus_watch_new                (int               fd,
+DBusWatch* _dbus_watch_new                (DBusPollable      fd,
                                            unsigned int      flags,
                                            dbus_bool_t       enabled,
                                            DBusWatchHandler  handler,
@@ -95,6 +95,17 @@ DBUS_PRIVATE_EXPORT
 void           _dbus_watch_set_oom_last_time  (DBusWatch               *watch,
                                                dbus_bool_t              oom);
 
+DBusSocket     _dbus_watch_get_socket         (DBusWatch               *watch);
+DBUS_PRIVATE_EXPORT
+DBusPollable   _dbus_watch_get_pollable       (DBusWatch               *watch);
+
+static inline void
+_dbus_clear_watch (DBusWatch **pointer_to_watch)
+{
+  _dbus_clear_pointer_impl (DBusWatch, pointer_to_watch,
+                            _dbus_watch_unref);
+}
+
 /** @} */
 
 DBUS_END_DECLS
