@@ -206,8 +206,7 @@ setup (Fixture *f,
       f->right_conn = dbus_bus_get_private (DBUS_BUS_SESSION, &f->e);
       test_assert_no_error (&f->e);
 
-      if (!test_connection_setup (f->ctx, f->right_conn))
-        g_error ("OOM");
+      test_connection_setup (f->ctx, f->right_conn);
     }
   else
     {
@@ -343,6 +342,8 @@ test_no_reply (Fixture *f,
   if (m == NULL)
     g_error ("OOM");
 
+  /* Not using test_main_context_call_and_wait() here because we need to
+   * do things with the right connection as a side-effect */
   if (!dbus_connection_send_with_reply (f->left_conn, m, &pc,
                                         DBUS_TIMEOUT_INFINITE) ||
       pc == NULL)
@@ -363,6 +364,7 @@ test_no_reply (Fixture *f,
         test_main_context_iterate (f->ctx, TRUE);
 
       dbus_connection_remove_filter (f->right_conn, echo_filter, f);
+      test_connection_shutdown (f->ctx, f->right_conn);
       dbus_connection_close (f->right_conn);
       dbus_clear_connection (&f->right_conn);
     }
@@ -395,7 +397,7 @@ test_creds (Fixture *f,
   const char *unique = dbus_bus_get_unique_name (f->left_conn);
   DBusMessage *m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
       DBUS_PATH_DBUS, DBUS_INTERFACE_DBUS, "GetConnectionCredentials");
-  DBusPendingCall *pc;
+  DBusMessage *reply = NULL;
   DBusMessageIter args_iter;
   DBusMessageIter arr_iter;
   DBusMessageIter pair_iter;
@@ -415,25 +417,12 @@ test_creds (Fixture *f,
         DBUS_TYPE_INVALID))
     g_error ("OOM");
 
-  if (!dbus_connection_send_with_reply (f->left_conn, m, &pc,
-                                        DBUS_TIMEOUT_USE_DEFAULT) ||
-      pc == NULL)
-    g_error ("OOM");
+  reply = test_main_context_call_and_wait (f->ctx, f->left_conn, m,
+      DBUS_TIMEOUT_USE_DEFAULT);
 
-  dbus_clear_message (&m);
+  g_assert_cmpstr (dbus_message_get_signature (reply), ==, "a{sv}");
 
-  if (dbus_pending_call_get_completed (pc))
-    test_pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
-                                          &m, NULL))
-    g_error ("OOM");
-
-  while (m == NULL)
-    test_main_context_iterate (f->ctx, TRUE);
-
-  g_assert_cmpstr (dbus_message_get_signature (m), ==, "a{sv}");
-
-  dbus_message_iter_init (m, &args_iter);
+  dbus_message_iter_init (reply, &args_iter);
   g_assert_cmpuint (dbus_message_iter_get_arg_type (&args_iter), ==,
       DBUS_TYPE_ARRAY);
   g_assert_cmpuint (dbus_message_iter_get_element_type (&args_iter), ==,
@@ -529,6 +518,10 @@ test_creds (Fixture *f,
           g_assert_not_reached ();
 #endif
         }
+      else if (g_str_has_prefix (name, DBUS_INTERFACE_CONTAINERS1 "."))
+        {
+          g_assert_not_reached ();
+        }
 
       dbus_message_iter_next (&arr_iter);
     }
@@ -545,8 +538,8 @@ test_creds (Fixture *f,
   g_assert (seen & SEEN_WINDOWS_SID);
 #endif
 
+  dbus_clear_message (&reply);
   dbus_clear_message (&m);
-  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -556,7 +549,7 @@ test_processid (Fixture *f,
   const char *unique = dbus_bus_get_unique_name (f->left_conn);
   DBusMessage *m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
       DBUS_PATH_DBUS, DBUS_INTERFACE_DBUS, "GetConnectionUnixProcessID");
-  DBusPendingCall *pc;
+  DBusMessage *reply = NULL;
   DBusError error = DBUS_ERROR_INIT;
   guint32 pid;
 
@@ -568,23 +561,10 @@ test_processid (Fixture *f,
         DBUS_TYPE_INVALID))
     g_error ("OOM");
 
-  if (!dbus_connection_send_with_reply (f->left_conn, m, &pc,
-                                        DBUS_TIMEOUT_USE_DEFAULT) ||
-      pc == NULL)
-    g_error ("OOM");
+  reply = test_main_context_call_and_wait (f->ctx, f->left_conn, m,
+      DBUS_TIMEOUT_USE_DEFAULT);
 
-  dbus_clear_message (&m);
-
-  if (dbus_pending_call_get_completed (pc))
-    test_pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
-                                          &m, NULL))
-    g_error ("OOM");
-
-  while (m == NULL)
-    test_main_context_iterate (f->ctx, TRUE);
-
-  if (dbus_set_error_from_message (&error, m))
+  if (dbus_set_error_from_message (&error, reply))
     {
       g_assert_cmpstr (error.name, ==, DBUS_ERROR_UNIX_PROCESS_ID_UNKNOWN);
 
@@ -595,11 +575,11 @@ test_processid (Fixture *f,
 
       dbus_error_free (&error);
     }
-  else if (dbus_message_get_args (m, &error,
+  else if (dbus_message_get_args (reply, &error,
         DBUS_TYPE_UINT32, &pid,
         DBUS_TYPE_INVALID))
     {
-      g_assert_cmpstr (dbus_message_get_signature (m), ==, "u");
+      g_assert_cmpstr (dbus_message_get_signature (reply), ==, "u");
       test_assert_no_error (&error);
 
       g_test_message ("GetConnectionUnixProcessID returned %u", pid);
@@ -617,8 +597,8 @@ test_processid (Fixture *f,
       g_error ("Unexpected error: %s: %s", error.name, error.message);
     }
 
+  dbus_clear_message (&reply);
   dbus_clear_message (&m);
-  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -627,7 +607,7 @@ test_canonical_path_uae (Fixture *f,
 {
   DBusMessage *m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
       DBUS_PATH_DBUS, DBUS_INTERFACE_DBUS, "UpdateActivationEnvironment");
-  DBusPendingCall *pc;
+  DBusMessage *reply = NULL;
   DBusMessageIter args_iter;
   DBusMessageIter arr_iter;
 
@@ -642,28 +622,15 @@ test_canonical_path_uae (Fixture *f,
       !dbus_message_iter_close_container (&args_iter, &arr_iter))
     g_error ("OOM");
 
-  if (!dbus_connection_send_with_reply (f->left_conn, m, &pc,
-                                        DBUS_TIMEOUT_USE_DEFAULT) ||
-      pc == NULL)
-    g_error ("OOM");
-
-  dbus_clear_message (&m);
-
-  if (dbus_pending_call_get_completed (pc))
-    test_pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
-                                          &m, NULL))
-    g_error ("OOM");
-
-  while (m == NULL)
-    test_main_context_iterate (f->ctx, TRUE);
+  reply = test_main_context_call_and_wait (f->ctx, f->left_conn, m,
+      DBUS_TIMEOUT_USE_DEFAULT);
 
   /* it succeeds */
-  g_assert_cmpint (dbus_message_get_type (m), ==,
+  g_assert_cmpint (dbus_message_get_type (reply), ==,
       DBUS_MESSAGE_TYPE_METHOD_RETURN);
 
+  dbus_clear_message (&reply);
   dbus_clear_message (&m);
-  dbus_clear_pending_call (&pc);
 
   /* Now try with the wrong object path */
   m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
@@ -680,30 +647,17 @@ test_canonical_path_uae (Fixture *f,
       !dbus_message_iter_close_container (&args_iter, &arr_iter))
     g_error ("OOM");
 
-  if (!dbus_connection_send_with_reply (f->left_conn, m, &pc,
-                                        DBUS_TIMEOUT_USE_DEFAULT) ||
-      pc == NULL)
-    g_error ("OOM");
-
-  dbus_clear_message (&m);
-
-  if (dbus_pending_call_get_completed (pc))
-    test_pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
-                                          &m, NULL))
-    g_error ("OOM");
-
-  while (m == NULL)
-    test_main_context_iterate (f->ctx, TRUE);
+  reply = test_main_context_call_and_wait (f->ctx, f->left_conn, m,
+      DBUS_TIMEOUT_USE_DEFAULT);
 
   /* it fails, yielding an error message with one string argument */
-  g_assert_cmpint (dbus_message_get_type (m), ==, DBUS_MESSAGE_TYPE_ERROR);
-  g_assert_cmpstr (dbus_message_get_error_name (m), ==,
+  g_assert_cmpint (dbus_message_get_type (reply), ==, DBUS_MESSAGE_TYPE_ERROR);
+  g_assert_cmpstr (dbus_message_get_error_name (reply), ==,
       DBUS_ERROR_ACCESS_DENIED);
-  g_assert_cmpstr (dbus_message_get_signature (m), ==, "s");
+  g_assert_cmpstr (dbus_message_get_signature (reply), ==, "s");
 
+  dbus_clear_message (&reply);
   dbus_clear_message (&m);
-  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -743,8 +697,10 @@ test_max_connections (Fixture *f,
     dbus_connection_close (failing_conn);
 
   dbus_clear_connection (&failing_conn);
+  test_connection_shutdown (f->ctx, third_conn);
   dbus_connection_close (third_conn);
   dbus_clear_connection (&third_conn);
+  dbus_error_free (&error);
 }
 
 static void
@@ -1045,13 +1001,9 @@ test_pending_fd_timeout (Fixture *f,
       test_progress ('.');
       test_main_context_iterate (f->ctx, TRUE);
 
-      /* It should take 0.5s to get disconnected, as configured in
-       * valid-config-files/pending-fd-timeout.conf; but this test
-       * might get starved by other processes running in parallel
-       * (particularly on shared CI systems), so we have to be a lot
-       * more generous. Allow up to 10 seconds. */
-      g_assert_cmpint (g_get_monotonic_time (), <=,
-                       start + (10 * G_USEC_PER_SEC));
+      /* It should take no longer than 500ms to get disconnected. We'll
+       * be generous and allow 1000ms. */
+      g_assert_cmpint (g_get_monotonic_time (), <=, start + G_USEC_PER_SEC);
     }
 
   g_object_unref (socket);
@@ -1207,7 +1159,7 @@ test_peer_get_machine_id (Fixture *f,
   char *what_i_think;
   const char *what_daemon_thinks;
   DBusMessage *m = NULL;
-  DBusPendingCall *pc = NULL;
+  DBusMessage *reply = NULL;
   DBusError error = DBUS_ERROR_INIT;
 
   if (f->skip)
@@ -1222,6 +1174,7 @@ test_peer_get_machine_id (Fixture *f,
           /* When running unit tests during make check or make installcheck,
            * tolerate this */
           g_test_skip ("Machine UUID not available");
+          dbus_error_free (&error);
           return;
         }
       else
@@ -1237,24 +1190,13 @@ test_peer_get_machine_id (Fixture *f,
                                     DBUS_INTERFACE_PEER,
                                     "GetMachineId");
 
-  if (m == NULL ||
-      !dbus_connection_send_with_reply (f->left_conn, m, &pc,
-                                        DBUS_TIMEOUT_USE_DEFAULT) ||
-      pc == NULL)
-    g_error ("OOM");
+  if (m == NULL)
+    test_oom ();
 
-  dbus_clear_message (&m);
+  reply = test_main_context_call_and_wait (f->ctx, f->left_conn, m,
+      DBUS_TIMEOUT_USE_DEFAULT);
 
-  if (dbus_pending_call_get_completed (pc))
-    test_pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
-                                          &m, NULL))
-    g_error ("OOM");
-
-  while (m == NULL)
-    test_main_context_iterate (f->ctx, TRUE);
-
-  if (!dbus_message_get_args (m, &error,
+  if (!dbus_message_get_args (reply, &error,
         DBUS_TYPE_STRING, &what_daemon_thinks,
         DBUS_TYPE_INVALID))
     g_error ("%s: %s", error.name, error.message);
@@ -1263,8 +1205,8 @@ test_peer_get_machine_id (Fixture *f,
   g_assert_nonnull (what_daemon_thinks);
   g_assert_cmpuint (strlen (what_daemon_thinks), ==, 32);
 
+  dbus_clear_message (&reply);
   dbus_clear_message (&m);
-  dbus_clear_pending_call (&pc);
   dbus_free (what_i_think);
 }
 
@@ -1272,9 +1214,8 @@ static void
 test_peer_ping (Fixture *f,
                 gconstpointer context)
 {
-  DBusMessage *m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
-      DBUS_PATH_DBUS, DBUS_INTERFACE_PEER, "Ping");
-  DBusPendingCall *pc = NULL;
+  DBusMessage *m = NULL;
+  DBusMessage *reply = NULL;
   DBusError error = DBUS_ERROR_INIT;
 
   if (f->skip)
@@ -1283,37 +1224,25 @@ test_peer_ping (Fixture *f,
   m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
       DBUS_PATH_DBUS, DBUS_INTERFACE_PEER, "Ping");
 
-  if (m == NULL ||
-      !dbus_connection_send_with_reply (f->left_conn, m, &pc,
-                                        DBUS_TIMEOUT_USE_DEFAULT) ||
-      pc == NULL)
-    g_error ("OOM");
+  if (m == NULL)
+    test_oom ();
 
-  dbus_clear_message (&m);
+  reply = test_main_context_call_and_wait (f->ctx, f->left_conn, m,
+      DBUS_TIMEOUT_USE_DEFAULT);
 
-  if (dbus_pending_call_get_completed (pc))
-    test_pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
-                                          &m, NULL))
-    g_error ("OOM");
-
-  while (m == NULL)
-    test_main_context_iterate (f->ctx, TRUE);
-
-  if (!dbus_message_get_args (m, &error, DBUS_TYPE_INVALID))
+  if (!dbus_message_get_args (reply, &error, DBUS_TYPE_INVALID))
     g_error ("%s: %s", error.name, error.message);
 
+  dbus_clear_message (&reply);
   dbus_clear_message (&m);
-  dbus_clear_pending_call (&pc);
 }
 
 static void
 test_get_invalid_path (Fixture *f,
                        gconstpointer context)
 {
-  DBusMessage *m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
-      "/", DBUS_INTERFACE_PROPERTIES, "Get");
-  DBusPendingCall *pc = NULL;
+  DBusMessage *m = NULL;
+  DBusMessage *reply = NULL;
   DBusError error = DBUS_ERROR_INIT;
   const char *iface = DBUS_INTERFACE_DBUS;
   const char *property = "Interfaces";
@@ -1321,45 +1250,36 @@ test_get_invalid_path (Fixture *f,
   if (f->skip)
     return;
 
+  m = dbus_message_new_method_call (DBUS_SERVICE_DBUS, "/",
+      DBUS_INTERFACE_PROPERTIES, "Get");
+
   if (m == NULL ||
       !dbus_message_append_args (m,
         DBUS_TYPE_STRING, &iface,
         DBUS_TYPE_STRING, &property,
-        DBUS_TYPE_INVALID) ||
-      !dbus_connection_send_with_reply (f->left_conn, m, &pc,
-                                        DBUS_TIMEOUT_USE_DEFAULT) ||
-      pc == NULL)
-    g_error ("OOM");
+        DBUS_TYPE_INVALID))
+    test_oom ();
 
-  dbus_clear_message (&m);
+  reply = test_main_context_call_and_wait (f->ctx, f->left_conn, m,
+      DBUS_TIMEOUT_USE_DEFAULT);
 
-  if (dbus_pending_call_get_completed (pc))
-    test_pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
-                                          &m, NULL))
-    g_error ("OOM");
-
-  while (m == NULL)
-    test_main_context_iterate (f->ctx, TRUE);
-
-  if (!dbus_set_error_from_message (&error, m))
+  if (!dbus_set_error_from_message (&error, reply))
     g_error ("Unexpected success");
 
   /* That object path does not have that interface */
   g_assert_cmpstr (error.name, ==, DBUS_ERROR_UNKNOWN_INTERFACE);
   dbus_error_free (&error);
 
+  dbus_clear_message (&reply);
   dbus_clear_message (&m);
-  dbus_clear_pending_call (&pc);
 }
 
 static void
 test_get_invalid_iface (Fixture *f,
                         gconstpointer context)
 {
-  DBusMessage *m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
-      DBUS_PATH_DBUS, DBUS_INTERFACE_PROPERTIES, "Get");
-  DBusPendingCall *pc = NULL;
+  DBusMessage *m = NULL;
+  DBusMessage *reply = NULL;
   DBusError error = DBUS_ERROR_INIT;
   const char *iface = "com.example.Nope";
   const char *property = "Whatever";
@@ -1367,44 +1287,35 @@ test_get_invalid_iface (Fixture *f,
   if (f->skip)
     return;
 
+  m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
+      DBUS_PATH_DBUS, DBUS_INTERFACE_PROPERTIES, "Get");
+
   if (m == NULL ||
       !dbus_message_append_args (m,
         DBUS_TYPE_STRING, &iface,
         DBUS_TYPE_STRING, &property,
-        DBUS_TYPE_INVALID) ||
-      !dbus_connection_send_with_reply (f->left_conn, m, &pc,
-                                        DBUS_TIMEOUT_USE_DEFAULT) ||
-      pc == NULL)
-    g_error ("OOM");
+        DBUS_TYPE_INVALID))
+    test_oom ();
 
-  dbus_clear_message (&m);
+  reply = test_main_context_call_and_wait (f->ctx, f->left_conn, m,
+      DBUS_TIMEOUT_USE_DEFAULT);
 
-  if (dbus_pending_call_get_completed (pc))
-    test_pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
-                                          &m, NULL))
-    g_error ("OOM");
-
-  while (m == NULL)
-    test_main_context_iterate (f->ctx, TRUE);
-
-  if (!dbus_set_error_from_message (&error, m))
+  if (!dbus_set_error_from_message (&error, reply))
     g_error ("Unexpected success");
 
   g_assert_cmpstr (error.name, ==, DBUS_ERROR_UNKNOWN_INTERFACE);
   dbus_error_free (&error);
 
+  dbus_clear_message (&reply);
   dbus_clear_message (&m);
-  dbus_clear_pending_call (&pc);
 }
 
 static void
 test_get_invalid (Fixture *f,
                   gconstpointer context)
 {
-  DBusMessage *m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
-      DBUS_PATH_DBUS, DBUS_INTERFACE_PROPERTIES, "Get");
-  DBusPendingCall *pc = NULL;
+  DBusMessage *m = NULL;
+  DBusMessage *reply = NULL;
   DBusError error = DBUS_ERROR_INIT;
   const char *iface = DBUS_INTERFACE_DBUS;
   const char *property = "Whatever";
@@ -1412,131 +1323,104 @@ test_get_invalid (Fixture *f,
   if (f->skip)
     return;
 
+  m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
+      DBUS_PATH_DBUS, DBUS_INTERFACE_PROPERTIES, "Get");
+
   if (m == NULL ||
       !dbus_message_append_args (m,
         DBUS_TYPE_STRING, &iface,
         DBUS_TYPE_STRING, &property,
-        DBUS_TYPE_INVALID) ||
-      !dbus_connection_send_with_reply (f->left_conn, m, &pc,
-                                        DBUS_TIMEOUT_USE_DEFAULT) ||
-      pc == NULL)
-    g_error ("OOM");
+        DBUS_TYPE_INVALID))
+    test_oom ();
 
-  dbus_clear_message (&m);
+  reply = test_main_context_call_and_wait (f->ctx, f->left_conn, m,
+      DBUS_TIMEOUT_USE_DEFAULT);
 
-  if (dbus_pending_call_get_completed (pc))
-    test_pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
-                                          &m, NULL))
-    g_error ("OOM");
-
-  while (m == NULL)
-    test_main_context_iterate (f->ctx, TRUE);
-
-  if (!dbus_set_error_from_message (&error, m))
+  if (!dbus_set_error_from_message (&error, reply))
     g_error ("Unexpected success");
 
   g_assert_cmpstr (error.name, ==, DBUS_ERROR_UNKNOWN_PROPERTY);
   dbus_error_free (&error);
 
+  dbus_clear_message (&reply);
   dbus_clear_message (&m);
-  dbus_clear_pending_call (&pc);
 }
 
 static void
 test_get_all_invalid_iface (Fixture *f,
                             gconstpointer context)
 {
-  DBusMessage *m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
-      DBUS_PATH_DBUS, DBUS_INTERFACE_PROPERTIES, "GetAll");
-  DBusPendingCall *pc = NULL;
+  DBusMessage *m = NULL;
+  DBusMessage *reply = NULL;
   DBusError error = DBUS_ERROR_INIT;
   const char *iface = "com.example.Nope";
 
   if (f->skip)
     return;
 
+  m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
+      DBUS_PATH_DBUS, DBUS_INTERFACE_PROPERTIES, "GetAll");
+
   if (m == NULL ||
       !dbus_message_append_args (m,
         DBUS_TYPE_STRING, &iface,
-        DBUS_TYPE_INVALID) ||
-      !dbus_connection_send_with_reply (f->left_conn, m, &pc,
-                                        DBUS_TIMEOUT_USE_DEFAULT) ||
-      pc == NULL)
-    g_error ("OOM");
+        DBUS_TYPE_INVALID))
+    test_oom ();
 
-  dbus_clear_message (&m);
+  reply = test_main_context_call_and_wait (f->ctx, f->left_conn, m,
+      DBUS_TIMEOUT_USE_DEFAULT);
 
-  if (dbus_pending_call_get_completed (pc))
-    test_pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
-                                          &m, NULL))
-    g_error ("OOM");
-
-  while (m == NULL)
-    test_main_context_iterate (f->ctx, TRUE);
-
-  if (!dbus_set_error_from_message (&error, m))
+  if (!dbus_set_error_from_message (&error, reply))
     g_error ("Unexpected success");
 
   g_assert_cmpstr (error.name, ==, DBUS_ERROR_UNKNOWN_INTERFACE);
   dbus_error_free (&error);
 
+  dbus_clear_message (&reply);
   dbus_clear_message (&m);
-  dbus_clear_pending_call (&pc);
 }
 
 static void
 test_get_all_invalid_path (Fixture *f,
                            gconstpointer context)
 {
-  DBusMessage *m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
-      "/", DBUS_INTERFACE_PROPERTIES, "GetAll");
-  DBusPendingCall *pc = NULL;
+  DBusMessage *m = NULL;
+  DBusMessage *reply = NULL;
   DBusError error = DBUS_ERROR_INIT;
   const char *iface = DBUS_INTERFACE_DBUS;
 
   if (f->skip)
     return;
 
+  m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
+      "/", DBUS_INTERFACE_PROPERTIES, "GetAll");
+
   if (m == NULL ||
       !dbus_message_append_args (m,
         DBUS_TYPE_STRING, &iface,
-        DBUS_TYPE_INVALID) ||
-      !dbus_connection_send_with_reply (f->left_conn, m, &pc,
-                                        DBUS_TIMEOUT_USE_DEFAULT) ||
-      pc == NULL)
-    g_error ("OOM");
+        DBUS_TYPE_INVALID))
+    test_oom ();
 
-  dbus_clear_message (&m);
+  reply = test_main_context_call_and_wait (f->ctx, f->left_conn, m,
+      DBUS_TIMEOUT_USE_DEFAULT);
 
-  if (dbus_pending_call_get_completed (pc))
-    test_pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
-                                          &m, NULL))
-    g_error ("OOM");
-
-  while (m == NULL)
-    test_main_context_iterate (f->ctx, TRUE);
-
-  if (!dbus_set_error_from_message (&error, m))
+  if (!dbus_set_error_from_message (&error, reply))
     g_error ("Unexpected success");
 
   /* That object path does not have that interface */
   g_assert_cmpstr (error.name, ==, DBUS_ERROR_UNKNOWN_INTERFACE);
   dbus_error_free (&error);
 
+  dbus_clear_message (&reply);
   dbus_clear_message (&m);
-  dbus_clear_pending_call (&pc);
 }
 
 static void
 test_set_invalid_iface (Fixture *f,
                         gconstpointer context)
 {
-  DBusMessage *m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
-      DBUS_PATH_DBUS, DBUS_INTERFACE_PROPERTIES, "Set");
-  DBusPendingCall *pc = NULL;
+  DBusMessage *m = NULL;
+  DBusMessage *reply = NULL;
   DBusError error = DBUS_ERROR_INIT;
   const char *iface = "com.example.Nope";
   const char *property = "Whatever";
@@ -1547,6 +1431,9 @@ test_set_invalid_iface (Fixture *f,
   if (f->skip)
     return;
 
+  m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
+      DBUS_PATH_DBUS, DBUS_INTERFACE_PROPERTIES, "Set");
+
   if (m == NULL ||
       !dbus_message_append_args (m,
         DBUS_TYPE_STRING, &iface,
@@ -1559,40 +1446,28 @@ test_set_invalid_iface (Fixture *f,
   if (!dbus_message_iter_open_container (&args_iter,
         DBUS_TYPE_VARIANT, "b", &var_iter) ||
       !dbus_message_iter_append_basic (&var_iter, DBUS_TYPE_BOOLEAN, &b) ||
-      !dbus_message_iter_close_container (&args_iter, &var_iter) ||
-      !dbus_connection_send_with_reply (f->left_conn, m, &pc,
-                                        DBUS_TIMEOUT_USE_DEFAULT) ||
-      pc == NULL)
-    g_error ("OOM");
+      !dbus_message_iter_close_container (&args_iter, &var_iter))
+    test_oom ();
 
-  dbus_clear_message (&m);
+  reply = test_main_context_call_and_wait (f->ctx, f->left_conn, m,
+      DBUS_TIMEOUT_USE_DEFAULT);
 
-  if (dbus_pending_call_get_completed (pc))
-    test_pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
-                                          &m, NULL))
-    g_error ("OOM");
-
-  while (m == NULL)
-    test_main_context_iterate (f->ctx, TRUE);
-
-  if (!dbus_set_error_from_message (&error, m))
+  if (!dbus_set_error_from_message (&error, reply))
     g_error ("Unexpected success");
 
   g_assert_cmpstr (error.name, ==, DBUS_ERROR_UNKNOWN_INTERFACE);
   dbus_error_free (&error);
 
+  dbus_clear_message (&reply);
   dbus_clear_message (&m);
-  dbus_clear_pending_call (&pc);
 }
 
 static void
 test_set_invalid_path (Fixture *f,
                        gconstpointer context)
 {
-  DBusMessage *m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
-      "/", DBUS_INTERFACE_PROPERTIES, "Set");
-  DBusPendingCall *pc = NULL;
+  DBusMessage *m = NULL;
+  DBusMessage *reply = NULL;
   DBusError error = DBUS_ERROR_INIT;
   const char *iface = DBUS_INTERFACE_DBUS;
   const char *property = "Interfaces";
@@ -1603,6 +1478,9 @@ test_set_invalid_path (Fixture *f,
   if (f->skip)
     return;
 
+  m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
+      "/", DBUS_INTERFACE_PROPERTIES, "Set");
+
   if (m == NULL ||
       !dbus_message_append_args (m,
         DBUS_TYPE_STRING, &iface,
@@ -1615,40 +1493,28 @@ test_set_invalid_path (Fixture *f,
   if (!dbus_message_iter_open_container (&args_iter,
         DBUS_TYPE_VARIANT, "b", &var_iter) ||
       !dbus_message_iter_append_basic (&var_iter, DBUS_TYPE_BOOLEAN, &b) ||
-      !dbus_message_iter_close_container (&args_iter, &var_iter) ||
-      !dbus_connection_send_with_reply (f->left_conn, m, &pc,
-                                        DBUS_TIMEOUT_USE_DEFAULT) ||
-      pc == NULL)
-    g_error ("OOM");
+      !dbus_message_iter_close_container (&args_iter, &var_iter))
+    test_oom ();
 
-  dbus_clear_message (&m);
+  reply = test_main_context_call_and_wait (f->ctx, f->left_conn, m,
+      DBUS_TIMEOUT_USE_DEFAULT);
 
-  if (dbus_pending_call_get_completed (pc))
-    test_pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
-                                          &m, NULL))
-    g_error ("OOM");
-
-  while (m == NULL)
-    test_main_context_iterate (f->ctx, TRUE);
-
-  if (!dbus_set_error_from_message (&error, m))
+  if (!dbus_set_error_from_message (&error, reply))
     g_error ("Unexpected success");
 
   g_assert_cmpstr (error.name, ==, DBUS_ERROR_UNKNOWN_INTERFACE);
   dbus_error_free (&error);
 
+  dbus_clear_message (&reply);
   dbus_clear_message (&m);
-  dbus_clear_pending_call (&pc);
 }
 
 static void
 test_set_invalid (Fixture *f,
                   gconstpointer context)
 {
-  DBusMessage *m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
-      DBUS_PATH_DBUS, DBUS_INTERFACE_PROPERTIES, "Set");
-  DBusPendingCall *pc = NULL;
+  DBusMessage *m = NULL;
+  DBusMessage *reply = NULL;
   DBusError error = DBUS_ERROR_INIT;
   const char *iface = DBUS_INTERFACE_DBUS;
   const char *property = "Whatever";
@@ -1659,6 +1525,9 @@ test_set_invalid (Fixture *f,
   if (f->skip)
     return;
 
+  m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
+      DBUS_PATH_DBUS, DBUS_INTERFACE_PROPERTIES, "Set");
+
   if (m == NULL ||
       !dbus_message_append_args (m,
         DBUS_TYPE_STRING, &iface,
@@ -1671,40 +1540,28 @@ test_set_invalid (Fixture *f,
   if (!dbus_message_iter_open_container (&args_iter,
         DBUS_TYPE_VARIANT, "b", &var_iter) ||
       !dbus_message_iter_append_basic (&var_iter, DBUS_TYPE_BOOLEAN, &b) ||
-      !dbus_message_iter_close_container (&args_iter, &var_iter) ||
-      !dbus_connection_send_with_reply (f->left_conn, m, &pc,
-                                        DBUS_TIMEOUT_USE_DEFAULT) ||
-      pc == NULL)
-    g_error ("OOM");
+      !dbus_message_iter_close_container (&args_iter, &var_iter))
+    test_oom ();
 
-  dbus_clear_message (&m);
+  reply = test_main_context_call_and_wait (f->ctx, f->left_conn, m,
+      DBUS_TIMEOUT_USE_DEFAULT);
 
-  if (dbus_pending_call_get_completed (pc))
-    test_pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
-                                          &m, NULL))
-    g_error ("OOM");
-
-  while (m == NULL)
-    test_main_context_iterate (f->ctx, TRUE);
-
-  if (!dbus_set_error_from_message (&error, m))
+  if (!dbus_set_error_from_message (&error, reply))
     g_error ("Unexpected success");
 
   g_assert_cmpstr (error.name, ==, DBUS_ERROR_UNKNOWN_PROPERTY);
   dbus_error_free (&error);
 
+  dbus_clear_message (&reply);
   dbus_clear_message (&m);
-  dbus_clear_pending_call (&pc);
 }
 
 static void
 test_set (Fixture *f,
           gconstpointer context)
 {
-  DBusMessage *m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
-      DBUS_PATH_DBUS, DBUS_INTERFACE_PROPERTIES, "Set");
-  DBusPendingCall *pc = NULL;
+  DBusMessage *m = NULL;
+  DBusMessage *reply = NULL;
   DBusError error = DBUS_ERROR_INIT;
   const char *iface = DBUS_INTERFACE_DBUS;
   const char *property = "Features";
@@ -1715,6 +1572,9 @@ test_set (Fixture *f,
   if (f->skip)
     return;
 
+  m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
+      DBUS_PATH_DBUS, DBUS_INTERFACE_PROPERTIES, "Set");
+
   if (m == NULL ||
       !dbus_message_append_args (m,
         DBUS_TYPE_STRING, &iface,
@@ -1727,31 +1587,20 @@ test_set (Fixture *f,
   if (!dbus_message_iter_open_container (&args_iter,
         DBUS_TYPE_VARIANT, "b", &var_iter) ||
       !dbus_message_iter_append_basic (&var_iter, DBUS_TYPE_BOOLEAN, &b) ||
-      !dbus_message_iter_close_container (&args_iter, &var_iter) ||
-      !dbus_connection_send_with_reply (f->left_conn, m, &pc,
-                                        DBUS_TIMEOUT_USE_DEFAULT) ||
-      pc == NULL)
-    g_error ("OOM");
+      !dbus_message_iter_close_container (&args_iter, &var_iter))
+    test_oom ();
 
-  dbus_clear_message (&m);
+  reply = test_main_context_call_and_wait (f->ctx, f->left_conn, m,
+      DBUS_TIMEOUT_USE_DEFAULT);
 
-  if (dbus_pending_call_get_completed (pc))
-    test_pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
-                                          &m, NULL))
-    g_error ("OOM");
-
-  while (m == NULL)
-    test_main_context_iterate (f->ctx, TRUE);
-
-  if (!dbus_set_error_from_message (&error, m))
+  if (!dbus_set_error_from_message (&error, reply))
     g_error ("Unexpected success");
 
   g_assert_cmpstr (error.name, ==, DBUS_ERROR_PROPERTY_READ_ONLY);
   dbus_error_free (&error);
 
+  dbus_clear_message (&reply);
   dbus_clear_message (&m);
-  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -1759,6 +1608,7 @@ check_features (DBusMessageIter *var_iter)
 {
   DBusMessageIter arr_iter;
   gboolean have_systemd_activation = FALSE;
+  gboolean have_header_filtering = FALSE;
 
   g_assert_cmpint (dbus_message_iter_get_arg_type (var_iter), ==,
       DBUS_TYPE_ARRAY);
@@ -1776,12 +1626,15 @@ check_features (DBusMessageIter *var_iter)
 
       g_test_message ("Feature: %s", feature);
 
-      if (g_strcmp0 (feature, "SystemdActivation") == 0)
+      if (g_strcmp0 (feature, "HeaderFiltering") == 0)
+        have_header_filtering = TRUE;
+      else if (g_strcmp0 (feature, "SystemdActivation") == 0)
         have_systemd_activation = TRUE;
 
       dbus_message_iter_next (&arr_iter);
     }
 
+  g_assert_true (have_header_filtering);
   /* We pass --systemd-activation to the daemon for this unit test on Unix
    * (it can only work in practice on Linux, but there's nothing
    * inherently Linux-specific about the protocol). */
@@ -1796,9 +1649,8 @@ static void
 test_features (Fixture *f,
                gconstpointer context)
 {
-  DBusMessage *m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
-      DBUS_PATH_DBUS, DBUS_INTERFACE_PROPERTIES, "Get");
-  DBusPendingCall *pc = NULL;
+  DBusMessage *m = NULL;
+  DBusMessage *reply = NULL;
   DBusMessageIter args_iter;
   DBusMessageIter var_iter;
   const char *iface = DBUS_INTERFACE_DBUS;
@@ -1807,28 +1659,20 @@ test_features (Fixture *f,
   if (f->skip)
     return;
 
+  m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
+      DBUS_PATH_DBUS, DBUS_INTERFACE_PROPERTIES, "Get");
+
   if (m == NULL ||
       !dbus_message_append_args (m,
         DBUS_TYPE_STRING, &iface,
         DBUS_TYPE_STRING, &features,
-        DBUS_TYPE_INVALID) ||
-      !dbus_connection_send_with_reply (f->left_conn, m, &pc,
-                                        DBUS_TIMEOUT_USE_DEFAULT) ||
-      pc == NULL)
-    g_error ("OOM");
+        DBUS_TYPE_INVALID))
+    test_oom ();
 
-  dbus_clear_message (&m);
+  reply = test_main_context_call_and_wait (f->ctx, f->left_conn, m,
+      DBUS_TIMEOUT_USE_DEFAULT);
 
-  if (dbus_pending_call_get_completed (pc))
-    test_pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
-                                          &m, NULL))
-    g_error ("OOM");
-
-  while (m == NULL)
-    test_main_context_iterate (f->ctx, TRUE);
-
-  if (!dbus_message_iter_init (m, &args_iter))
+  if (!dbus_message_iter_init (reply, &args_iter))
     g_error ("Reply has no arguments");
 
   g_assert_cmpint (dbus_message_iter_get_arg_type (&args_iter), ==,
@@ -1840,8 +1684,8 @@ test_features (Fixture *f,
   if (dbus_message_iter_next (&args_iter))
     g_error ("Reply has too many arguments");
 
+  dbus_clear_message (&reply);
   dbus_clear_message (&m);
-  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -1901,9 +1745,8 @@ static void
 test_interfaces (Fixture *f,
                  gconstpointer context)
 {
-  DBusMessage *m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
-      DBUS_PATH_DBUS, DBUS_INTERFACE_PROPERTIES, "Get");
-  DBusPendingCall *pc = NULL;
+  DBusMessage *m = NULL;
+  DBusMessage *reply = NULL;
   DBusMessageIter args_iter;
   DBusMessageIter var_iter;
   const char *iface = DBUS_INTERFACE_DBUS;
@@ -1912,28 +1755,20 @@ test_interfaces (Fixture *f,
   if (f->skip)
     return;
 
+  m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
+      DBUS_PATH_DBUS, DBUS_INTERFACE_PROPERTIES, "Get");
+
   if (m == NULL ||
       !dbus_message_append_args (m,
         DBUS_TYPE_STRING, &iface,
         DBUS_TYPE_STRING, &ifaces,
-        DBUS_TYPE_INVALID) ||
-      !dbus_connection_send_with_reply (f->left_conn, m, &pc,
-                                        DBUS_TIMEOUT_USE_DEFAULT) ||
-      pc == NULL)
-    g_error ("OOM");
+        DBUS_TYPE_INVALID))
+    test_oom ();
 
-  dbus_clear_message (&m);
+  reply = test_main_context_call_and_wait (f->ctx, f->left_conn, m,
+      DBUS_TIMEOUT_USE_DEFAULT);
 
-  if (dbus_pending_call_get_completed (pc))
-    test_pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
-                                          &m, NULL))
-    g_error ("OOM");
-
-  while (m == NULL)
-    test_main_context_iterate (f->ctx, TRUE);
-
-  if (!dbus_message_iter_init (m, &args_iter))
+  if (!dbus_message_iter_init (reply, &args_iter))
     g_error ("Reply has no arguments");
 
   if (dbus_message_iter_get_arg_type (&args_iter) != DBUS_TYPE_VARIANT)
@@ -1945,17 +1780,16 @@ test_interfaces (Fixture *f,
   if (dbus_message_iter_next (&args_iter))
     g_error ("Reply has too many arguments");
 
+  dbus_clear_message (&reply);
   dbus_clear_message (&m);
-  dbus_clear_pending_call (&pc);
 }
 
 static void
 test_get_all (Fixture *f,
               gconstpointer context)
 {
-  DBusMessage *m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
-      DBUS_PATH_DBUS, DBUS_INTERFACE_PROPERTIES, "GetAll");
-  DBusPendingCall *pc = NULL;
+  DBusMessage *m = NULL;
+  DBusMessage *reply = NULL;
   DBusMessageIter args_iter;
   DBusMessageIter arr_iter;
   DBusMessageIter pair_iter;
@@ -1967,27 +1801,19 @@ test_get_all (Fixture *f,
   if (f->skip)
     return;
 
+  m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
+      DBUS_PATH_DBUS, DBUS_INTERFACE_PROPERTIES, "GetAll");
+
   if (m == NULL ||
       !dbus_message_append_args (m,
         DBUS_TYPE_STRING, &iface,
-        DBUS_TYPE_INVALID) ||
-      !dbus_connection_send_with_reply (f->left_conn, m, &pc,
-                                        DBUS_TIMEOUT_USE_DEFAULT) ||
-      pc == NULL)
-    g_error ("OOM");
+        DBUS_TYPE_INVALID))
+    test_oom ();
 
-  dbus_clear_message (&m);
+  reply = test_main_context_call_and_wait (f->ctx, f->left_conn, m,
+      DBUS_TIMEOUT_USE_DEFAULT);
 
-  if (dbus_pending_call_get_completed (pc))
-    test_pending_call_store_reply (pc, &m);
-  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
-                                          &m, NULL))
-    g_error ("OOM");
-
-  while (m == NULL)
-    test_main_context_iterate (f->ctx, TRUE);
-
-  dbus_message_iter_init (m, &args_iter);
+  dbus_message_iter_init (reply, &args_iter);
   g_assert_cmpuint (dbus_message_iter_get_arg_type (&args_iter), ==,
       DBUS_TYPE_ARRAY);
   g_assert_cmpuint (dbus_message_iter_get_element_type (&args_iter), ==,
@@ -2027,8 +1853,8 @@ test_get_all (Fixture *f,
   if (dbus_message_iter_next (&args_iter))
     g_error ("Reply has too many arguments");
 
+  dbus_clear_message (&reply);
   dbus_clear_message (&m);
-  dbus_clear_pending_call (&pc);
 }
 
 #define DESIRED_RLIMIT 65536
@@ -2040,7 +1866,6 @@ test_fd_limit (Fixture *f,
 {
 #ifdef HAVE_PRLIMIT
   struct rlimit lim;
-  struct rlimit new_limit;
   const struct passwd *pwd = NULL;
 #endif
 
@@ -2068,7 +1893,7 @@ test_fd_limit (Fixture *f,
     }
 
   if (prlimit (getpid (), RLIMIT_NOFILE, NULL, &lim) < 0)
-    g_error ("get prlimit (self): %s", g_strerror (errno));
+    g_error ("prlimit(): %s", g_strerror (errno));
 
   g_test_message ("our RLIMIT_NOFILE: rlim_cur: %ld, rlim_max: %ld",
                   (long) lim.rlim_cur, (long) lim.rlim_max);
@@ -2080,33 +1905,8 @@ test_fd_limit (Fixture *f,
       return;
     }
 
-  new_limit = lim;
-  new_limit.rlim_cur = DESIRED_RLIMIT;
-  new_limit.rlim_max = DESIRED_RLIMIT;
-
-  /* Try to increase the rlimit ourselves. If we're root in an
-   * unprivileged Linux container, then we won't have CAP_SYS_RESOURCE
-   * and this will fail with EPERM. If so, the dbus-daemon wouldn't be
-   * able to increase its rlimit either. */
-  if (prlimit (getpid (), RLIMIT_NOFILE, &new_limit, NULL) < 0)
-    {
-      gchar *message;
-
-      message = g_strdup_printf ("Cannot test, we cannot change the rlimit so "
-                                 "presumably neither can the dbus-daemon: %s",
-                                 g_strerror (errno));
-      g_test_skip (message);
-      g_free (message);
-      return;
-    }
-
-  /* Immediately put our original limit back so it won't interfere with
-   * subsequent tests. This should always succeed. */
-  if (prlimit (getpid (), RLIMIT_NOFILE, &lim, NULL) < 0)
-    g_error ("Cannot restore our original limits: %s", g_strerror (errno));
-
   if (prlimit (f->daemon_pid, RLIMIT_NOFILE, NULL, &lim) < 0)
-    g_error ("get prlimit (dbus-daemon): %s", g_strerror (errno));
+    g_error ("prlimit(): %s", g_strerror (errno));
 
   g_test_message ("dbus-daemon's RLIMIT_NOFILE: rlim_cur: %ld, rlim_max: %ld",
                   (long) lim.rlim_cur, (long) lim.rlim_max);
@@ -2130,10 +1930,15 @@ teardown (Fixture *f,
   g_clear_error (&f->ge);
 
   if (f->left_conn != NULL)
-    dbus_connection_close (f->left_conn);
+    {
+      test_connection_shutdown (f->ctx, f->left_conn);
+      dbus_connection_close (f->left_conn);
+    }
 
   if (f->right_conn != NULL)
     {
+      GList *link;
+
       if (f->right_conn_echo)
         {
           dbus_connection_remove_filter (f->right_conn, echo_filter, f);
@@ -2146,9 +1951,12 @@ teardown (Fixture *f,
           f->right_conn_hold = FALSE;
         }
 
-      g_queue_foreach (&f->held_messages, (GFunc) dbus_message_unref, NULL);
+      for (link = f->held_messages.head; link != NULL; link = link->next)
+        dbus_message_unref (link->data);
+
       g_queue_clear (&f->held_messages);
 
+      test_connection_shutdown (f->ctx, f->right_conn);
       dbus_connection_close (f->right_conn);
     }
 
@@ -2254,6 +2062,8 @@ int
 main (int argc,
     char **argv)
 {
+  int ret;
+
   test_init (&argc, &argv);
 
   g_test_add ("/echo/session", Fixture, NULL, setup, test_echo, teardown);
@@ -2331,5 +2141,7 @@ main (int argc,
               setup, test_fd_limit, teardown);
 #endif
 
-  return g_test_run ();
+  ret = g_test_run ();
+  dbus_shutdown ();
+  return ret;
 }
