@@ -256,8 +256,7 @@ setup_connection (Fixture *f,
   dbus_server_set_new_connection_function (f->server,
       new_conn_cb, f, NULL);
 
-  if (!test_server_setup (f->loop, f->server))
-    g_error ("failed to set up server");
+  test_server_setup (f->loop, f->server);
 
   address = dbus_server_get_address (f->server);
   g_assert (address != NULL);
@@ -266,8 +265,7 @@ setup_connection (Fixture *f,
   g_assert (f->connection != NULL);
   dbus_free (address);
 
-  if (!test_connection_setup (f->loop, f->connection))
-    g_error ("failed to set up connection");
+  test_connection_setup (f->loop, f->connection);
 
   while (f->server_connection == NULL)
     _dbus_loop_iterate (f->loop, TRUE);
@@ -343,6 +341,7 @@ test_connection (Fixture *f,
 
   /* Destroy the connection. This should be the last-unref. */
   g_assert (!f->last_unref);
+  test_connection_shutdown (f->loop, f->connection);
   dbus_connection_close (f->connection);
   dbus_connection_unref (f->connection);
   f->connection = NULL;
@@ -426,7 +425,7 @@ test_server (Fixture *f,
 
   /* Destroy the server. This should be the last-unref. */
   g_assert (!f->last_unref);
-  dbus_server_disconnect (f->server);
+  test_server_shutdown (f->loop, f->server);
   dbus_server_unref (f->server);
   f->server = NULL;
   g_assert (f->last_unref);
@@ -595,12 +594,14 @@ teardown (Fixture *f,
 {
   if (f->server_connection != NULL)
     {
+      test_connection_shutdown (f->loop, f->server_connection);
       dbus_connection_close (f->server_connection);
       dbus_connection_unref (f->server_connection);
     }
 
   if (f->connection != NULL)
     {
+      test_connection_shutdown (f->loop, f->connection);
       dbus_connection_close (f->connection);
       dbus_connection_unref (f->connection);
     }
@@ -624,6 +625,8 @@ int
 main (int argc,
     char **argv)
 {
+  int ret;
+
   test_init (&argc, &argv);
 
   g_test_add ("/refs/connection", Fixture, NULL, setup_connection,
@@ -635,5 +638,7 @@ main (int argc,
   g_test_add ("/refs/server", Fixture, NULL, setup,
       test_server, teardown);
 
-  return g_test_run ();
+  ret = g_test_run ();
+  dbus_shutdown ();
+  return ret;
 }
