@@ -29,6 +29,7 @@
 
 #include <dbus/dbus.h>
 
+#include <gio/gio.h>
 #include <glib.h>
 
 #include "test-utils.h"
@@ -74,9 +75,16 @@ gchar *test_get_dbus_daemon (const gchar *config_file,
 
 DBusConnection *test_connect_to_bus (TestMainContext *ctx,
     const gchar *address);
-DBusConnection *test_connect_to_bus_as_user (TestMainContext *ctx,
+DBusConnection *test_try_connect_to_bus (TestMainContext *ctx,
+    const gchar *address,
+    GError **error);
+DBusConnection *test_try_connect_to_bus_as_user (TestMainContext *ctx,
     const char *address,
-    TestUser user);
+    TestUser user,
+    GError **error);
+GDBusConnection *test_try_connect_gdbus_as_user (const char *address,
+    TestUser user,
+    GError **error);
 
 void test_kill_pid (GPid pid);
 
@@ -91,6 +99,13 @@ void test_mkdir (const gchar *path, gint mode);
 
 void test_timeout_reset (guint factor);
 
+void test_oom (void) _DBUS_GNUC_NORETURN;
+
+DBusMessage *test_main_context_call_and_wait (TestMainContext *ctx,
+    DBusConnection *connection,
+    DBusMessage *call,
+    int timeout);
+
 #if !GLIB_CHECK_VERSION(2, 44, 0)
 #define g_steal_pointer(x) backported_g_steal_pointer (x)
 /* A simplified version of g_steal_pointer without type-safety. */
@@ -108,4 +123,8 @@ backported_g_steal_pointer (gpointer pointer_to_pointer)
 
 gboolean test_check_tcp_works (void);
 
+void test_store_result_cb (GObject *source_object,
+                           GAsyncResult *result,
+                           gpointer user_data);
+
 #endif
